--[[

	Name:		MobHealth2
	Author:		Wyv	& Skeeve
	Description:	Displays health	value for mobs.
	Original version by	Telo.
]]


-- global variable holding all current target data for entire MobInfo2 AddOn
MI2_Target = { };
MI2_LastTargetIdx = nil;

local COLOR_BLUE	= "|c009090FF";
local COLOR_WHITE	= "|c00FFFFFF";
local TITLE		= "|c003060FFMobHealth2"..COLOR_WHITE;
local lOldFontId = 0;
local lOldFontSize = 0;


--------------------------------------------------------------------------------------------------
-- external	functions for macros / scripts
--------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- MobHealth_GetTargetCurHP()  
--
-- Return current health points	value for the current target as	an integer
-- value. Return nil if	there is no	current	target.
--
-- Example:
--	 local targetCurrentHealth = MobHealth_GetTargetCurHP();
--	 if	 targetCurrentHealth  then
--		.......
--	 end
-----------------------------------------------------------------------------
function MobHealth_GetTargetCurHP()
	return MI2_Target.curHealth;
end	 --	of MobHealth_GetTargetCurHP()


-----------------------------------------------------------------------------
-- MobHealth_GetTargetMaxHP()  
--
-- Return maximum health points	value for the current target as	an integer
-- value. Return nil if	there is no	current	target.
--
-- Example:
--	 local targetMaxHealth = MobHealth_GetTargetMaxHP();
--	 if	 targetMaxHealth  then
--		.......
--	 end
-----------------------------------------------------------------------------
function MobHealth_GetTargetMaxHP()
	return MI2_Target.maxHealth;
end	 --	of MobHealth_GetTargetMaxHP()


-----------------------------------------------------------------------------
-- MobHealth_PPP( index	)  
--
-- Return the Points-Per-Percent (PPP) value for a Mob identified by its index.
-- The index is	the	concatination of the Mob name and the Mob level	(see
-- example below). 0 is	returned if	the	PPP	value is not available for
-- the given index.	The	example	also shows how to calculate	the	actual
-- health points from the health percentage	and	the	PPP	value
--
-- Example:
--	  local	name  =	UnitName("target");
--	  local	level =	UnitLevel("target");
--	  local	index =	name..":"..level;
--	  local	ppp	= MobHealth_PPP( index );
--	  local	healthPercent =	UnitHealth("target");
--	  local	curHealth =	math.floor(	healthPercent *	ppp	+ 0.5);
--	  local	maxHealth =	math.floor(	100	* ppp +	0.5);
-----------------------------------------------------------------------------
function MobHealth_PPP(	index )
	if	index and MobHealthDB[index]  then
		local s, e;
		local pts;
		local pct;
		
		s, e, pts, pct = string.find(MobHealthDB[index], "^(%d+)/(%d+)$");
		if ( pts and pct ) then
			pts	= pts +	0;
			pct	= pct +	0;
			if ( pct ~=	0 )	then
				return pts / pct;
			end
		end
	end
	return 0;
end

--------------------------------------------------------------------------------------------------
-- Internal	functions
--	 - all MobHealthDB access functions	are	left intact	for	compatiblity
--	 - MobHealth is	no longer local	(but it's not a	recomended way to access mob health	- even tho
--	   it's	better than	using MobHealthDB directly)
--------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- MobHealth_GetPts()
--
-- return pts value for given mob index within given database
-----------------------------------------------------------------------------
local function MobHealth_GetPts( database, index )
	if database[index] then
		local s, e,	val	= string.find(database[index], "^(%d+)/");
		if	val	 then
			return val + 0;
		end
	end
	return 0;
end


-----------------------------------------------------------------------------
-- MobHealth_GetPct()
--
-- return pct value for given mob index within given database
-----------------------------------------------------------------------------
local function MobHealth_GetPct( database, index )
	if database[index] then
		local s, e,	val	= string.find(database[index], "/(%d+)$");
		if	val	 then
			return val + 0;
		end
	end
	return 0;
end  -- MobHealth_GetPct()


-----------------------------------------------------------------------------
-- MobHealth_Set()
--
-- store pct and pts values for a given mob index in the given database
-----------------------------------------------------------------------------
local function MobHealth_Set( database, index, pts, pct)
	if pts or pct then
		database[index] = (pts or 0).."/"..(pct or 0);
	else
		database[index] = nil;
	end
end  -- MobHealth_Set()


-----------------------------------------------------------------------------
-- MobHealth_Display()
--
-- display the values and percentage for health	/ mana in target frame
-----------------------------------------------------------------------------
local function MobHealth_Display( )
	local healthText = "";
	local manaText = "";

	if MI2_Target.targetWithHealth then
		-- create health text if showing health is enabled
		if  MobInfoConfig.TargetHealth == 1 then
			if MI2_Target.curHealth then
				healthText = string.format("%d/%d", MI2_Target.curHealth, MI2_Target.maxHealth );
			else
				healthText = "-/-";
			end
		end

		-- create health percent text if showing percent is enabled
		if	MobInfoConfig.HealthPercent	== 1 and MI2_Target.curPercent then
			if  healthText == ""  then
				healthText = string.format("%d%%", MI2_Target.curPercent );
			else
				healthText = healthText..string.format(" (%d%%)", MI2_Target.curPercent );
			end
		end

		-- create mana text based on mana show flags
		local mana	  =	UnitMana("target");
		local maxmana =	UnitManaMax("target");
		if	MobInfoConfig.TargetMana ==	1 and maxmana > 0 then
			manaText = string.format("%d/%d", mana, maxmana );
		end
		if	MobInfoConfig.ManaPercent == 1	and	 maxmana > 0  then
			if  manaText == ""  then
				manaText = string.format("%d%%", math.floor(100.0 * mana / maxmana));
			else
				manaText = manaText..string.format(" (%d%%)", math.floor(100.0 * mana / maxmana));
			end
		end
	end

	MI2_MobHealthText:SetText(healthText);
	MI2_MobManaText:SetText(manaText);
end	 --	MobHealth_Display()


-----------------------------------------------------------------------------
-- MI2_MobHealth_SetFont()
--
-- set new font	for	display	of health /	mana in	target frame
-----------------------------------------------------------------------------
local function MI2_MobHealth_SetFont( fontId, fontSize )
	local	fontName;

	if  fontId ~=	lOldFontId	or	fontSize ~=	lOldFontSize  then
		lOldFontId = fontId;
		lOldFontSize = fontSize;

		-- select font name	to use	
		if	fontId == 1	 then
			fontName = "Fonts\\ARIALN.TTF";  -- NumberFontNormal
		elseif	fontId == 2	 then
			fontName = "Fonts\\FRIZQT__.TTF";	 --	GameFontNormal
		else
			fontName = "Fonts\\MORPHEUS.TTF";	 --	ItemTextFontNormal
		end
		-- LedMirage 1.12.2006		[ 1 of 1]
		-- set font	for	health and mana	text
		--MI2_MobHealthText:SetFont( fontName, fontSize );
		--MI2_MobManaText:SetFont( fontName, fontSize);		
		MI2_MobHealthText:SetFont( "Interface\\Addons\\Insomniax_Core\\Fonts\\IX_Font_Bold.ttf", 10 );
		MI2_MobManaText:SetFont( "Interface\\Addons\\Insomniax_Core\\Fonts\\IX_Font_Bold.ttf", 10 );
	end
  
end	 --	of MI2_MobHealth_SetFont()


-----------------------------------------------------------------------------
-- MI2_MobHealth_SetPos()
--
-- set position	and	font for mob health/mana texts
-----------------------------------------------------------------------------
function MI2_MobHealth_SetPos( )
	local font;

	-- set poition for health	and	mana text
	MI2_MobHealthText:SetPoint( "TOP", "TargetFrameHealthBar", "BOTTOM", MobInfoConfig.HealthPosX, MobInfoConfig.HealthPosY );
	MI2_MobManaText:SetPoint( "TOP", "TargetFrameManaBar", "BOTTOM", MobInfoConfig.ManaPosX, MobInfoConfig.ManaPosY );

	-- update	font ID	and	font size
	MI2_MobHealth_SetFont( MobInfoConfig.TargetFont, MobInfoConfig.TargetFontSize	);

	-- redisplay health /	mana values
	MobHealth_Display();
end	 --	of MI2_MobHealth_SetPos()


function MI2_MobHealth_VariablesLoaded()
	if not MobHealthDB then
		MobHealthDB	= {	};
	else
		local index, value;
		for	index, value in	MobHealthDB	do
			-- Convert the old-style data into the newer, more compact form
			if(	type(value)	== "table" ) then
				MobHealth_Set( MobHealthDB, index, value.damPts, value.damPct );
			end
		end
	end

	if not MobHealthPlayerDB then
		MobHealthPlayerDB =	{};
	end

end


function MI2_MobHealth_Reset()
	MI2_MobHealth_ClearTargetData();
	MobHealthDB	= {};
	MobHealthPlayerDB =	{};
end


local function MobHealth_SaveTargetData()
	if MI2_Target and MI2_Target.PPP > 0 and MI2_Target.totalPercent < 10000 then
		MobHealth_Set( MI2_Target.database, MI2_Target.index, MI2_Target.totalDamage, MI2_Target.totalPercent );
	end
end


-----------------------------------------------------------------------------
-- MI2_MobHealth_ClearTargetData()
--
-- Clear mob health data for current target
-----------------------------------------------------------------------------
function MI2_MobHealth_ClearTargetData()
	if MI2_Target.index and MI2_Target.database then
		MI2_Target.database[MI2_Target.index] = nil;
		MI2_Target.index = nil;
		MI2_OnTargetChanged();
	end
end  -- MI2_MobHealth_ClearTargetData()


-----------------------------------------------------------------------------
-- MI2_MH_CalculateHealth()
--
-- Recalculate all health values depending on target health type (recentage
-- or absolute) and health calculation base values (totalDmg, totalPercent)
-----------------------------------------------------------------------------
local function MI2_MH_CalculateHealth( maxHealthOverride )
	if MI2_Target.targetWithHealth then
		MI2_Target.curMana = UnitMana("target");

		if MI2_Target.index then
			MI2_Target.curPercent = UnitHealth("target");
			
			if MI2_Target.totalPercent > 0 then
				MI2_Target.PPP = MI2_Target.totalDamage / MI2_Target.totalPercent;
				MI2_Target.curHealth = math.floor(MI2_Target.curPercent * MI2_Target.PPP + 0.5);
				
				if  MobInfoConfig.StableMax == 0 or MI2_Target.firstBattle or maxHealthOverride then
					MI2_Target.maxHealth =	math.floor((100 * MI2_Target.PPP) + 0.5);
				end
			end
		else
			MI2_Target.curHealth = UnitHealth("target");
			MI2_Target.curPercent = math.floor(100.0 * MI2_Target.curHealth / MI2_Target.maxHealth + 0.5);
		end
	end
end  -- MI2_MH_CalculateHealth()


--------------------------------------------------------------------------------------------------
-- Event handlers
--------------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------
-- MI2_MH_EventUnitHealth()
--
-- Event handler for the "UNIT_HEALTH" event. This handler will
-- process health number that WoW gives us for the current target. Combining
-- the health value with the damage done to the current target and with the
-- current health percentage of the target allows us to calculate the PPP
-- value (Point Per Percent) for the current target. The PPP value can then
-- be used to calculate a health value from a given health percentage.
-----------------------------------------------------------------------------
function MI2_MH_EventUnitHealth()
	local returnValue = false;

	if arg1 == "target" and MI2_Target.name then
		local health = UnitHealth("target");
		
		if MI2_Target.index then
			local change = MI2_Target.curPercent - health;
			local damage = MI2_Target.tempDamage;

if MI2_Debug > 1 then chattext( "DBG: change="..change.."("..MI2_Target.curPercent..","..health..")"..", dmg="..damage..", totP="..MI2_Target.totalPercent..", totD="..MI2_Target.totalDamage ); end
			
			if change > 0 and damage > 0 and health > 3 then
				MI2_Target.totalDamage = MI2_Target.totalDamage + damage;
				MI2_Target.totalPercent = MI2_Target.totalPercent + change;
				
				if MI2_Target.firstBattle then
					-- this	is needed for other	addons reading directly	from DB
					MobHealth_SaveTargetData()
				end
			end

			MI2_Target.tempDamage = 0;
			returnValue = true;
		end

		MI2_MH_CalculateHealth();
		MobHealth_Display( );
	end
	
	return returnValue;
end  -- MI2_MH_EventUnitHealth


-----------------------------------------------------------------------------
-- MI2_MH_EventUnitCombat()
--
-- Event handler for the "UNIT_COMBAT" event. This handler will
-- accumulate all damage done to the current target, if the current
-- target is one whose health we want to calculate.
-----------------------------------------------------------------------------
function MI2_MH_EventUnitCombat()

if MI2_Debug > 1 then chattext( "DBG: UNIT_COMBAT: arg1="..arg1..", arg4="..arg4 ); end

	if MI2_Target.index then
		-- record damage done  to target
		if arg1 == "target" then
			MI2_Target.tempDamage = MI2_Target.tempDamage + arg4;
		end
	end
end  -- MI2_MH_EventUnitCombat()


-----------------------------------------------------------------------------
-- MI2_OnTargetChanged()
--
-- Event handler for the "PLAYER_TARGET_CHANGED" event. This handler will
-- fill the global variable "MI2_Target" with all the data that MobInfo2
-- needs to know about the current target.
-----------------------------------------------------------------------------
function MI2_OnTargetChanged()
	local name = UnitName("target");

	-- record dps for old target, clenup mobinfo data
	if  MI2_Target.mobIndex then
		if MI2_Target.FightStartTime then
			MI2_RecordDps( MI2_Target.mobIndex, MI2_Target.FightEndTime - MI2_Target.FightStartTime, MI2_Target.FightDamage );
		end
		MI2_CleanupMobData( MI2_Target.mobIndex );
		MI2_LastTargetIdx = MI2_Target.mobIndex;
	end

	-- if we have old target we should	update it's	HP in database
	if MI2_Target.index then
		MobHealth_SaveTargetData();
	end
	
	MI2_Target = { };

	if name then
		MI2_Target.name = name;
		MI2_Target.level = UnitLevel("target");
		MI2_Target.class = UnitClass("target");
		MI2_Target.maxHealth = UnitHealthMax("target");
		MI2_Target.maxMana = UnitManaMax("target");
		MI2_Target.tempDamage = 0;
		MI2_Target.PPP = 0;
		
		local index = name..":"..MI2_Target.level;

		-- health calculation and health tracking initialisation
		if not UnitIsDead("target") then
			if MI2_Target.maxHealth == 100 and (UnitCanAttack("player","target") or UnitIsPlayer("target")) then
				-- choose appropriate health database and matching index
				MI2_Target.targetWithHealth = true;
				if  UnitIsPlayer("target")  then
					MI2_Target.index = name;
					MI2_Target.database = MobHealthPlayerDB;
				else
					MI2_Target.index = index;
					MI2_Target.database = MobHealthDB;
				end
			
				MI2_Target.totalDamage = MobHealth_GetPts(MI2_Target.database,MI2_Target.index);
				MI2_Target.totalPercent = MobHealth_GetPct(MI2_Target.database,MI2_Target.index);

				if MI2_Target.totalPercent <= 0 then
					MI2_Target.firstBattle = true; -- first battle	with mob/player
				end
			elseif MI2_Target.maxHealth ~= 100 then
				MI2_Target.targetWithHealth = true;
			end

			MI2_MH_CalculateHealth( true );
		end
		
		-- special mob index for MobInfo tooltip data collection
		if UnitCreatureType("target") and not UnitIsPlayer("target") then
			MI2_Target.mobIndex = index;
		end
	end

-- debugprofilestart() - Startet einen Timer fr das Profiling whrend Debuggingvorgngen.
-- millis = debugprofilestop()

-- debugging output describing current selected target
if MI2_Debug > 0 then		
if MI2_Target.name then
chattext( "name=["..MI2_Target.name.."], L"..MI2_Target.level..", class=["..MI2_Target.class.."], last=["..(MI2_LastTargetIdx or "nil").."]" );
chattext( "health=["..(MI2_Target.curHealth or "nil").."/"..MI2_Target.maxHealth.."], mana=["..(MI2_Target.curMana or "nil").."/"..MI2_Target.maxMana.."], tmpDmg=["..MI2_Target.tempDamage.."], ppp=["..MI2_Target.PPP.."]" );
if MI2_Target.index then
chattext( "idx=["..MI2_Target.index.."], totalDmg=["..MI2_Target.totalDamage.."], totalPct=["..MI2_Target.totalPercent.."], first=["..tostring(MI2_Target.firstBattle).."]" );
end 
else
chattext( "target=[nil], last=["..(MI2_LastTargetIdx or "nil").."]" );
end end

--	if name and UnitCanAttack("player","target") and not UnitIsDead("target") and not UnitIsFriend("player","target") then
--	if name and not UnitIsDead("target") and (UnitHealthMax("target") == 100)
--	   and (UnitCanAttack("player","target") or UnitIsPlayer("target")) then
--		-- target is attackable, it is not a friend, and it is alive
--	end

	-- update options dialog if shown
    if  frmMIConfig:IsVisible()  then
		MI2_DbOptionsFrameOnShow();
	end

	-- update mob health display
	MobHealth_Display();
end  -- MI2_OnTargetChanged()

