--
-- MI2_Search.lua
--
-- MobInfo module to control the Mob database search feature.
-- Search option settings and actual search algorithm are located in here.
--

--
-- start up defaults for search options settings
MI2_SearchOptions = {};
MI2_SearchOptions.MinLevel = 1;
MI2_SearchOptions.MaxLevel = 65;
MI2_SearchOptions.Normal = 1;
MI2_SearchOptions.Elite = 0;
MI2_SearchOptions.Boss = 0;
MI2_SearchOptions.MinLoots = 2;
MI2_SearchOptions.MobName = "";
MI2_SearchOptions.ResultLen = 10;

local MI2_SearchResultList = {};


-----------------------------------------------------------------------------
-- MI2_SearchOptionsOnShow()
--
-- OnShow event handler for search options page
-- Write current search option settings into the search option controls.
-- Validate all values and update colors accordingly.
-- Allow Search only if all search options are valid.
-----------------------------------------------------------------------------
function MI2_SearchOptionsOnShow()
	MI2_OptSearchMinLevel:SetText( tostring(MI2_SearchOptions.MinLevel) );
	MI2_OptSearchMaxLevel:SetText( tostring(MI2_SearchOptions.MaxLevel) );
	MI2_OptSearchMinLoots:SetText( tostring(MI2_SearchOptions.MinLoots) );
	MI2_OptSearchMobName:SetText( MI2_SearchOptions.MobName );
	MI2_OptSearchResultLen:SetText( tostring(MI2_SearchOptions.ResultLen) );

	MI2_OptSearchNormal:SetChecked( MI2_SearchOptions.Normal );
	MI2_OptSearchElite:SetChecked( MI2_SearchOptions.Elite );
	MI2_OptSearchBoss:SetChecked( MI2_SearchOptions.Boss );
end  -- of MI2_SearchOptionsOnShow()


-----------------------------------------------------------------------------
-- MI2_ValidateSearchOptions()
--
-- Validate all values and update colors accordingly.
-- Allow Search only if all search options are valid.
-----------------------------------------------------------------------------
local function MI2_ValidateSearchOptions()
	local optionsValid = true;

	if MI2_SearchOptions.MinLevel < 1 or MI2_SearchOptions.MinLevel > MI2_SearchOptions.MaxLevel then
		MI2_OptSearchMinLevel:SetTextColor( 1.0, 0.0, 0.0 );
		optionsValid = false;
	else
		MI2_OptSearchMinLevel:SetTextColor( 1.0, 1.0, 1.0 );
	end

	if MI2_SearchOptions.MaxLevel < 1 or MI2_SearchOptions.MaxLevel > 65 then
		MI2_OptSearchMaxLevel:SetTextColor( 1.0, 0.0, 0.0 );
		optionsValid = false;
	else
		MI2_OptSearchMaxLevel:SetTextColor( 1.0, 1.0, 1.0 );
	end

	if MI2_SearchOptions.MinLoots < 1 or MI2_SearchOptions.MinLoots > 100 then
		MI2_OptSearchMinLoots:SetTextColor( 1.0, 0.0, 0.0 );
		optionsValid = false;
	else
		MI2_OptSearchMinLoots:SetTextColor( 1.0, 1.0, 1.0 );
	end

	if MI2_SearchOptions.ResultLen < 1 or MI2_SearchOptions.ResultLen > 99 then
		MI2_OptSearchResultLen:SetTextColor( 1.0, 0.0, 0.0 );
		optionsValid = false;
	else
		MI2_OptSearchResultLen:SetTextColor( 1.0, 1.0, 1.0 );
	end

	if MI2_SearchOptions.Normal == 0 and MI2_SearchOptions.Elite == 0 and MI2_SearchOptions.Boss == 0 then
		optionsValid = false;
	end
	
	if optionsValid then
		MI2_OptSearchByValue:Enable();
	else
		MI2_OptSearchByValue:Disable();
	end
end  -- of MI2_ValidateSearchOptions()


-----------------------------------------------------------------------------
-- MI2_SearchCheckboxClicked()
--
-- OnClicked event handler for checkboxes on search options page
-- Store the checkbox state in the corresponding search options variable.
-----------------------------------------------------------------------------
function MI2_SearchCheckboxClicked()
	local checkboxName = this:GetName();
	local optionName = string.sub( checkboxName, 14 );
	local optionValue = this:GetChecked() or 0;

	MI2_SearchOptions[optionName] = optionValue;
	MI2_ValidateSearchOptions();
end  -- of MI2_SearchCheckboxClicked()


-----------------------------------------------------------------------------
-- MI2_SearchValueChanged()
--
-- OnChar event handler for editbox controls on search options page
-- This handler is called whenever the contents of an EditBox control changes.
-- It gets the new value and stores it in the corresponding search options
-- variable
-----------------------------------------------------------------------------
function MI2_SearchValueChanged()
	local editboxName = this:GetName();
	local optionName = string.sub( editboxName, 14 );
	local optionValue = tonumber(this:GetText()) or 0;

	MI2_SearchOptions[optionName] = optionValue;
	MI2_ValidateSearchOptions();
end  -- of MI2_SearchValueChanged()


-----------------------------------------------------------------------------
-- MI2_SearchTextChanged()
--
-- OnChar event handler for textual editbox controls on search options page
-- This handler is called whenever the contents of an EditBox control changes.
-- It gets the new value and stores it in the corresponding search options
-- variable
-----------------------------------------------------------------------------
function MI2_SearchTextChanged()
	local editboxName = this:GetName();
	local optionName = string.sub( editboxName, 14 );

	MI2_SearchOptions[optionName] = this:GetText();
	MI2_ValidateSearchOptions();
end  -- of MI2_SearchTextChanged()


-----------------------------------------------------------------------------
-- MI2_SlashAction_SearchByValue()
--
-- Search for most valuable mobs by comparing the mobs average total value.
-- The function creates a list of the N most valuable mobs and prints this
-- list to the chat log window.
-----------------------------------------------------------------------------
local function MI2_SearchByValue()
	local insertPos = 0;
	local value, rank, level;
	local numFound = 0;
	local resultMaxSize = MI2_SearchOptions.ResultLen;

	-- initialise search result list  
	MI2_SearchResultList = {};
	for i = 1, resultMaxSize do
		MI2_SearchResultList[i] = { rank=0 };
	end

	-- created a sorted list of the most valuable mobs
	-- by looping across all mobs in the MobInfo database
	-- and MI2_SearchOptions.Normal == 1 or (MI2_SearchOptions.Normal == 1 and mobData.mt 
	for index, mobData in MobInfoDB do
		local mobName, mobLevel = MI2_GetIndexComponents( index );
		local levelOk = mobLevel and (mobLevel >= MI2_SearchOptions.MinLevel and mobLevel <= MI2_SearchOptions.MaxLevel or mobLevel < 0);
		local lootsOk = mobData.lt and mobData.lt >= MI2_SearchOptions.MinLoots;
		local typeOk = (MI2_SearchOptions.Normal == 1 and not mobData.mt) or (MI2_SearchOptions.Elite == 1 and tonumber(mobData.mt) == 2) or (MI2_SearchOptions.Boss == 1 and tonumber(mobData.mt) == 3);
		local nameOk = string.find( index, MI2_SearchOptions.MobName ) ~= nil;

		if levelOk and lootsOk and typeOk and nameOk then
			numFound = numFound + 1;

			-- calculate bonus factor based on mob level and loots items quality
			local bonusFactor = mobLevel / 20;
			value = (mobData.cp or 0) + (mobData.iv or 0);
			rank = value + ((mobData.r2 or 0) * 200 * bonusFactor) + ((mobData.r3 or 0) * 1000 * bonusFactor) + ((mobData.r4 or 0) * 2000 * bonusFactor);
			rank = ceil( rank / mobData.lt );

			-- check if this mob enters the result list, find mob insert position
			if rank > MI2_SearchResultList[1].rank then
				insertPos = 1;
				for i = 2, resultMaxSize do
					if rank > MI2_SearchResultList[i].rank then
						MI2_SearchResultList[i-1] = MI2_SearchResultList[i];
						insertPos = i;
					else
						break;
					end
				end

				-- insert mob at correct sorted position and store all info we need for printing the result list
				MI2_SearchResultList[insertPos] = {};
				MI2_SearchResultList[insertPos].idx	= index;
				MI2_SearchResultList[insertPos].val	= ceil( value / mobData.lt );
				MI2_SearchResultList[insertPos].rank= rank;
				MI2_SearchResultList[insertPos].mob	= mobData;
			end
		end
	end

	chattext( "MobInfo Search: found total of "..numFound.." Mobs that match search criteria" );

end  -- MI2_SlashAction_SearchByValue()



-----------------------------------------------------------------------------
-- MI2_DisplaySearchResult()
--
-- Search for most valuable mobs by comparing the mobs average total value.
-- The function creates a list of the N most valuable mobs and prints this
-- list to the chat log window.
-----------------------------------------------------------------------------
function MI2_DisplaySearchResult()
	local resultText;
	local maxResults = MI2_SearchOptions.ResultLen;
	
	for i = 1, maxResults do
		if MI2_SearchResultList[i].rank > 0 then
			resultText = string.format( "%2d:  v=%s, q=(%s,%s,%s),  l=%d,  %s", 
				maxResults - i + 1,
				copper2text(MI2_SearchResultList[i].val)..mifontWhite,
				mifontGreen..(MI2_SearchResultList[i].mob.r2 or "0"),
				mifontBlue..(MI2_SearchResultList[i].mob.r3 or "0"),
				mifontMageta..(MI2_SearchResultList[i].mob.r4 or "0")..mifontWhite,
				MI2_SearchResultList[i].mob.lt,
				MI2_SearchResultList[i].idx );

			chattext( resultText );
		end
	end
end  -- MI2_DisplaySearchResult()


function MI2_SlashAction_SearchByValue()
--	chattext( "DBG: MinLevel = "..MI2_SearchOptions.MinLevel );
--	chattext( "DBG: MaxLevel = "..MI2_SearchOptions.MaxLevel );
--	chattext( "DBG: mob type Normal = "..MI2_SearchOptions.Normal );
--	chattext( "DBG: mob type Elite = "..MI2_SearchOptions.Elite );
--	chattext( "DBG: mob type Boss = "..MI2_SearchOptions.Boss );
--	chattext( "DBG: MinLoots = "..MI2_SearchOptions.MinLoots );
--	chattext( "DBG: MobName = "..MI2_SearchOptions.MobName );
--	chattext( "DBG: ResultLen = "..MI2_SearchOptions.ResultLen );

	MI2_SearchByValue();
	MI2_DisplaySearchResult()

end

