--
-- MobInfo2.lua
--
-- Main module of MobInfo-2 AddOn
-- Version:  2.52
--
-- MobInfo-2 is a World of Warcraft AddOn that provides you with useful
-- additional information about Mobs (ie. opponents/monsters). It adds
-- new information to the game's Tooltip when you hover with your mouse
-- over a mob. It also adds a numeric display of the Mobs health
-- and mana (current and max) to the Mob target frame.
--
-- MobInfo-2 is the continuation of the original "MobInfo" by Dizzarian,
-- combined with the original "MobHealth2" by Wyv. Both Dizzarian and
-- Wyv sadly no longer play WoW and stopped maintaining their AddOns.
-- I have "inhereted" MobInfo from Dizzarian and MobHealth-2 from Wyv
-- and now continue to update and improve the united result.
--

-- global vars
MI2_Debug = 0;  -- 0=no debug info, 1=minimal debug info, 2=extensive debug info, 3=more extensive+event info

-- local vars
local lMi2GameTooltip_OnEvent_Orig;
local lLastRecordedKill = "";
local lLastLooted = "";
local lLastLoot1 = "";
local lLastLoot2 = "";
local lLastLootNum = 1;
local lPlayer;
local lMiHealthLine, lMiManaLine;
local lMiMouseoverIndex = "";
local lEventHandlers = {};



-----------------------------------------------------------------------------
-- MI2_GetDamageData()
--
-- read damage data from mob database
-- returns: dl (damage lower), du (damage upper), dps (damage per second)
-----------------------------------------------------------------------------
local function MI2_GetDamageData( index, lPlayer )
	local a, b, dl, du, dps = 0,0,0,0,0;
	if MobInfoDB[index][lPlayer] then
		if MobInfoDB[index][lPlayer].dd then
			a, b, dl, du, dps = string.find( MobInfoDB[index][lPlayer].dd, "^(%d+)/(%d+)/(%d+)$" );
		end
	end
	return dl+0, du+0, dps+0;
end  -- MI2_GetDamageData


-----------------------------------------------------------------------------
-- MI2_GetMobData( mobName, mobLevel [, unitId] )
--
-- Get and return all the data that MobInfo knows about a given mob.
-- This is an externally available interface function that can be
-- called by other AddOns to access MobInfo data. It should be fast,
-- efficient, and easy to use
--
-- The data describing a Mob is returned in table form as described below.
--
-- To identify the mob you must supply its name and level. You can
-- optionally supply a "unitId" to get additional info:
--   mobName : name of mob, eg. "Forest Lurker"
--   mobLevel : mob level as integer number
--   unitId : optional WoW unit identification, should be either
--            "target" or "mouseover"
--
-- Examples:
--    A.   mobData = MI2_GetMobData( "Forest Lurker", 10 );
--    B.   mobData = MI2_GetMobData( "Forest Lurker", 10, "target" );
--
-- Return Value:
-- The return value is a LUA table with one table entry for each value that
-- MobInfo can know about a Mob. Note that table entries exist ONLY if the
-- corresponding value has actually been collected for the given Mob.
-- Unrecorded values do NOT exist in the table and thus evaluate to a NIL
-- expression.
--
-- Values you can get without "unitId" (as per Example A above):
--    mobData.healthMax  :  health maximum
--    mobData.xp         :  experience value
--    mobData.mob2Level  :  number of kills to next level
--    mobData.kills      :  number of times current player has killed this mob
--    mobData.minDamage  :  minimum damage done by mob
--    mobData.maxDamage  :  maximum damage done by mob
--    mobData.dps        :  dps of Mon against current player
--    mobData.loots      :  number of times this mob has been looted
--    mobData.emptyLoots :  number of times this mob gave empty loot
--    mobData.clothCount :  number of times this mob gave cloth loot
--    mobData.copper     :  total money loot of this mob as copper amount
--    mobData.itemValue  :  total item value loot of this mob as copper amount
--    mobData.r1         :  number of rarity 1 loot items (grey)
--    mobData.r2         :  number of rarity 2 loot items (white)
--    mobData.r3         :  number of rarity 3 loot items (green)
--    mobData.r4         :  number of rarity 4 loot items (blue)
--    mobData.r5         :  number of rarity 5 loot items (purple)
--
-- Additional values you will get with "unitId" (as per Example B above):
--    mobData.class      :  class of mob as localized text
--    mobData.healthCur  :  current health of given unit
--    mobData.manaCur    :  current mana of given unit
--    mobData.manaMax    :  maximum mana for given unit
--
-- Code Example:
--    
--    local mobData = MI2_GetMobData( "Forest Lurker", 10 );
--    
--    if mobData.xp then
--        DEFAULT_CHAT_FRAME:AddMessage( "XP = "..mobData.xp ); 
--    end
--    
--    if mobData.copper and mobData.loots then
--        local avgLoot = mobData.copper / mobData.loots;
--        DEFAULT_CHAT_FRAME:AddMessage( "average loot = "..avgLoot ); 
--    end
-----------------------------------------------------------------------------
function MI2_GetMobData( mobName, mobLevel, unitId )
	local mobData = {};
	local mobIndex = mobName..":"..mobLevel;

    -- get mobs PPP and calculate max health
	local mobPPP = MobHealth_PPP(mobIndex);
	if mobPPP <= 0 then mobPPP = 1; end;
	mobData.healthMax = floor(mobPPP * 100 + 0.5);

	if MI2_Debug > 2 then chattext( "M2DBG: MI2_GetMobData: name=["..mobName.."], level="..mobLevel..", exists: "..tostring(MobInfoDB[mobIndex] ~= nil) ); end

	-- obtain unit specific values if unitId is given
	if unitId then
		mobData.class = UnitClass(unitId);
		mobData.healthCur = floor(mobPPP * UnitHealth(unitId) + 0.5);
		mobData.manaCur = UnitMana( unitId );
		mobData.manaMax = UnitManaMax( unitId );
	end

	-- exit here if mob does not exist in DB, set color only if mob exists
	local mobInfo = MobInfoDB[mobIndex];
	if  not mobInfo then return mobData; end
	mobData.color = GetDifficultyColor( mobLevel );

	-- get xp (experience) and xp based values
	if mobInfo.xp then
		mobData.xp = mobInfo.xp;
		local xpCurrent = UnitXP("player") + mobData.xp;
		local xpToLevel = UnitXPMax("player") - xpCurrent;
		mobData.mob2Level = ceil(abs(xpToLevel / mobData.xp))+1;
	end

	-- get mob kills for current player
    if mobInfo[lPlayer] and mobInfo[lPlayer].kl then
		if mobInfo[lPlayer].kl > 0 then
			mobData.kills = mobInfo[lPlayer].kl;
		else
			mobInfo[lPlayer].kl = nil;
		end
	end

	-- get damage data for mob (always assign all 3 values)
	local minDmg, maxDmg, dps = MI2_GetDamageData( mobIndex, lPlayer );
	if (minDmg > 0 and maxDmg > 0) or dps > 0 then
		mobData.minDamage, mobData.maxDamage, mobData.dps = minDmg, maxDmg, dps;
	end

	if mobInfo.lt then mobData.loots = mobInfo.lt; end
	if mobInfo.el then mobData.emptyLoots = mobInfo.el; end
    if mobInfo.cc then mobData.clothCount = mobInfo.cc; end
    if mobInfo.cp then mobData.copper = mobInfo.cp; end
    if mobInfo.iv then mobData.itemValue = mobInfo.iv; end
	if mobInfo.r0 then mobData.r1 = mobInfo.r0; end;
	if mobInfo.r1 then mobData.r2 = mobInfo.r1; end;
	if mobInfo.r2 then mobData.r3 = mobInfo.r2; end;
	if mobInfo.r3 then mobData.r4 = mobInfo.r3; end;
	if mobInfo.r4 then mobData.r5 = mobInfo.r4; end;

	return mobData;
end


-----------------------------------------------------------------------------
-- chattext()
--
-- spits out msg to the chat channel. used in debuging
-----------------------------------------------------------------------------
function chattext(txt)
  if( DEFAULT_CHAT_FRAME ) then
    DEFAULT_CHAT_FRAME:AddMessage(txt);
  end
end  -- of chattext()


-----------------------------------------------------------------------------
-- MI2_CheckForSeparateMobHealth()
--
-- Detect the presence of separate MobHelath AddOns and disable the internal
-- MobHealth functionality if found
-----------------------------------------------------------------------------
function MI2_CheckForSeparateMobHealth()
  if  MobHealth_OnLoad  then
      if  MobInfoConfig.DisableHealth ~= 2  then
        MobInfoConfig.DisableHealth = 2;
        UIErrorsFrame:AddMessage( MI_TXT_MH_DISABLED, 0.0, 1.0, 1.0, 1.0, 20 );
      end
      MI2_MobHealthFrame:Hide();
  else
    if  MobInfoConfig.DisableHealth ~= 0  then
      MobInfoConfig.DisableHealth = 0;
    end
  end
end  -- MI2_CheckForSeparateMobHealth


-----------------------------------------------------------------------------
-- MI2_InitOptions()
--
-- initialize MobInfo configuration options
-- this takes into account new options that have been added to MobInfo
-- in the course of developement
-----------------------------------------------------------------------------
local function MI2_InitOptions()

  -- initialize MobInfoConfig
  if  not MobInfoConfig  or  not MobInfoConfig.ShowLoots  then	
    MobInfoConfig = { };  
    MI2_SlashAction_Default();  
  end  

  -- older options still missing on some installations
  if  not MobInfoConfig.SaveAllValues	then  MobInfoConfig.SaveAllValues = 1;		end
  if  not MobInfoConfig.ShowBlankLines	then  MobInfoConfig.ShowBlankLines = 1;		end
  if  not MobInfoConfig.ClearOnExit		then  MobInfoConfig.ClearOnExit = 0;		end
  if  not MobInfoConfig.CustomTracks	then  MobInfoConfig.CustomTracks = {};		end

  -- new config values since 1.7 (create them and set proper defaults)
  if  not MobInfoConfig.ShowDamage		then  MobInfoConfig.ShowDamage = 1;			end
  if  not MobInfoConfig.ShowMana		then  MobInfoConfig.ShowMana = 1;			end
  if  not MobInfoConfig.ShowEmpty		then  MobInfoConfig.ShowEmpty = 0;			end
  if  not MobInfoConfig.CombinedMode	then  MobInfoConfig.CombinedMode = 0;		end
  if  not MobInfoConfig.ShowCombined	then  MobInfoConfig.ShowCombined = 1;		end
  if  not MobInfoConfig.KeypressMode	then  MobInfoConfig.KeypressMode = 0;		end
  if  not MobInfoConfig.StableMax		then  MobInfoConfig.StableMax = 0;			end
  if  not MobInfoConfig.TargetHealth	then  MobInfoConfig.TargetHealth = 1;		end
  if  not MobInfoConfig.TargetMana		then  MobInfoConfig.TargetMana = 1;			end
  if  not MobInfoConfig.HealthPercent	then  MobInfoConfig.HealthPercent = 1;		end
  if  not MobInfoConfig.ManaPercent		then  MobInfoConfig.ManaPercent = 1;		end
  if  not MobInfoConfig.HealthPosX		then  MobInfoConfig.HealthPosX = -7;		end
  if  not MobInfoConfig.HealthPosY		then  MobInfoConfig.HealthPosY = 11;		end
  if  not MobInfoConfig.ManaPosX		then  MobInfoConfig.ManaPosX = -7;			end
  if  not MobInfoConfig.ManaPosY		then  MobInfoConfig.ManaPosY = 11;			end
  if  not MobInfoConfig.TargetFont		then  MobInfoConfig.TargetFont = 2;			end
  if  not MobInfoConfig.TargetFontSize	then  MobInfoConfig.TargetFontSize = 10;	end
  if  not MobInfoConfig.SavePlayerHp	then  MobInfoConfig.SavePlayerHp = 0;		end
  if  not MobInfoConfig.DisableMobInfo	then  MobInfoConfig.DisableMobInfo = 0;		end
  if  not MobInfoConfig.CompactMode		then  MobInfoConfig.CompactMode = 0;		end
  

  -- former option "HealthOff" has been renamed to "DisableHealth"
  if  not MobInfoConfig.DisableHealth  then  
    MobInfoConfig.DisableHealth = (MobInfoConfig.HealthOff or 0);
  end

  -- config values that no longer exist
  if  MobInfoConfig.HealthOff     then  MobInfoConfig.HealthOff = nil;  end
  if  MobInfoConfig.ManaDistance  then  MobInfoConfig.ManaDistance = nil; end
  if  MobInfoConfig.ShowPercent   then  MobInfoConfig.ShowPercent = nil; end
  
end  -- of MI2_InitOptions()


-----------------------------------------------------------------------------
-- MI2_CleanupDatabases()
--
-- Cleanup for MobInfo database. This function corrects bugs in the
-- MobInfo database and applies some changes that have been made to
-- record database entires.
-----------------------------------------------------------------------------
local function MI2_CleanupDatabases()
	if not MobInfoConfig.MobDbVersion or MobInfoConfig.MobDbVersion < 2 then
		MobInfoConfig.MobDbVersion = 2;
		for mobIndex, mobInfo in MobInfoDB do
			-- fixes for some very common 0 value database bugs
			if mobInfo.lt and mobInfo.lt <= 0 then mobInfo.lt = nil; end
			if mobInfo.cp and mobInfo.cp <= 0 then mobInfo.cp = nil; end
			if mobInfo.iv and mobInfo.iv <= 0 then mobInfo.iv = nil; end
			if mobInfo.el and mobInfo.el <= 0 then mobInfo.el = nil; end
			if mobInfo.cc and mobInfo.cc <= 0 then mobInfo.cc = nil; end

			-- search for player entries in mobInfo record (only player entries are tables)
			for entryName, entryData in mobInfo do
				if type(entryData) == "table" then
					local playerName = entryName;
					-- convert old dl,du entries into the new dd (damage data) entry
					if mobInfo[playerName] and mobInfo[playerName].dl then
						local dl = mobInfo[playerName].dl;
						local du = mobInfo[playerName].du;
						mobInfo[playerName].dl = nil;
						mobInfo[playerName].du = nil;
						if not mobInfo[playerName].dd then
							mobInfo[playerName].dd = dl.."/"..du.."/"..0;
						end
					end
					-- remove player entries without data
					if not mobInfo[playerName].dd and not mobInfo[playerName].kl then
						mobInfo[playerName] = nil;
					end;
				end
			end
		end
	end
end  -- MI2_CleanupDatabases()


-----------------------------------------------------------------------------
-- MI2_MobInfo_Initialize()
--
-- main global initialization function, this is called as the handler
-- for the "VARIABLES_LOADED" event
-----------------------------------------------------------------------------
local function MI2_MobInfo_Initialize()  

	-- initialize "MobInfoConfig" data structure (main MobInfo config options)
	MI2_InitOptions();

	-- clear MobInfo DB if it does not exist or if started with "ClearOnExit" set
	-- clear player health DB if corresponding save flag not set
	if  not MobInfoDB  or  MobInfoConfig.ClearOnExit == 1  then
  		MobInfoDB = { };
	end
	if  MobInfoConfig.SavePlayerHp == 0  then
		MobHealthPlayerDB = {};
	end

	-- register with all AddOn managers that MobInfo attempts to support
	-- currently that is: myAddons, KHAOS (mainly for Cosmos), EARTH (originally for Cosmos)
	MI2_RegisterWithAddonManagers();

	-- obtain player name and realm name  
	lPlayer = GetCVar( "realmName" )..':'..UnitName("player");

	-- check for presence of separate interferring MobHealth AddOn
	-- initialize built-in MobHealth if not disabled
	MI2_CheckForSeparateMobHealth();
	if  MobInfoConfig.DisableHealth < 2  then
		MI2_MobHealth_VariablesLoaded();
		MI2_MobHealth_SetPos();
	end

	-- ensure that MobHealthFrame get set correctly (if we have to set it for compatibility)
	if  MobHealthFrame == "MI2"  then
		MobHealthFrame = MI2_MobHealthFrame;
	end
	      
	-- check whether we have to use our own copy of the Auctioneer item value
	-- database because the Auctioneer AddOn is not installed
	if   not Auctioneer_GetVendorSellPrice   then
		if MI2_Debug > 0 then chattext( "M2DBG: loading built-in item sell value table" ); end
		MobInfo_Auctioneer_BuildBaseData();
	end

	-- setup a confirmation dialog for critical configuration options
	StaticPopupDialogs["MOBINFO_CONFIRMATION"] = {
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		showAlert = 1,
		timeout = 0,
		exclusive = 1,
		whileDead = 1,
		interruptCinematic = 1
	};

	-- setup event handling for the entire AddOn (MobInfo and MobHealth)
	lMi2GameTooltip_OnEvent_Orig = GameTooltip:GetScript("OnEvent");
	GameTooltip:SetScript( "OnEvent", MI2_GameTooltip_OnEvent );
	MI2_UpdateMobInfoState();

	-- initialize slash commands processing
	MI2_SlashInit();
	
	-- cleanup for all databases
	MI2_CleanupDatabases();

	chattext("MobInfo-2 "..miVersionNo.." Loaded. http://dizzarian.com ".. mifontGreen..'/mobinfo2 or /mi2 for help');
end  -- of MI2_MobInfo_Initialize()


-----------------------------------------------------------------------------
-- Retrieves # of times you looted the mob from MobInfoDb
-----------------------------------------------------------------------------
function GetMobLoots(index)
  if MobInfoDB[index] then
    if MobInfoDB[index].lt then
      return MobInfoDB[index].lt;
    end
  end
  return 0;
end


-----------------------------------------------------------------------------
-- Retrieves mob gold from MobInfoDb
-----------------------------------------------------------------------------
function GetMobCopper(index)
  if MobInfoDB[index] then
      if MobInfoDB[index].cp then
        return MobInfoDB[index].cp;
      end
  end
  return 0;
end


-----------------------------------------------------------------------------
-- Retrieves mob Item Value from MobInfoDb
-----------------------------------------------------------------------------
function GetMobIV(index)
  if MobInfoDB[index] then
      if MobInfoDB[index].iv then
        return MobInfoDB[index].iv;
      end
  end
  return 0;
end


-----------------------------------------------------------------------------
-- Retrieves mob Item Value from MobInfoDb
-----------------------------------------------------------------------------
function GetMobRarity(index, rareIndex)
  if MobInfoDB[index] then
      if MobInfoDB[index].r0 and rareIndex==1 then
        return MobInfoDB[index].r0;
      end
      if MobInfoDB[index].r1 and rareIndex==2 then
        return MobInfoDB[index].r1;
      end
      if MobInfoDB[index].r2 and rareIndex==3 then
        return MobInfoDB[index].r2;
      end
      if MobInfoDB[index].r3 and rareIndex==4 then
        return MobInfoDB[index].r3;
      end
      if MobInfoDB[index].r4 and rareIndex==5 then
        return MobInfoDB[index].r4;
      end
  end
  return 0;
end


-----------------------------------------------------------------------------
-- GetMobCloth()
--
-- Get cloth loot counter for Mob
-----------------------------------------------------------------------------
function GetMobCloth(index)
  if MobInfoDB[index] then
      if MobInfoDB[index].cc then
        return MobInfoDB[index].cc;
      end
  end
  return 0;
end  -- of GetMobCloth()


-----------------------------------------------------------------------------
-- GetMobCustom()
--
-- Get custom tracked item
-----------------------------------------------------------------------------
function GetMobCustom(index, item)
  if MobInfoDB[index] then
      if MobInfoDB[index][item] then
        return MobInfoDB[index][item];
      end
  end
  return 0;
end  -- of GetMobCustom()


-----------------------------------------------------------------------------
-- Add custom tracked item
-----------------------------------------------------------------------------
function MobInfoDbSetCustom(index, customitem, amt)
  if index then
    if MobInfoDB[index] == nil then
      MobInfoDB[index] = { };
      MobInfoDB[index][customitem] = 0;
    end 

    local c = GetMobCustom(index, customitem);
    if (c + amt) > 0 then
      MobInfoDB[index][customitem] = (c + amt);      
    else
      MobInfoDB[index][customitem] = 0;
    end       
  end
end;


-----------------------------------------------------------------------------
-- MI2_MobDbCheck()
--
-- check if given Mob exists in DB, create a new Mob record in DB if not
-----------------------------------------------------------------------------
local function MI2_MobDbCheck( index, playerName )
	if MobInfoDB[index] == nil then
		MobInfoDB[index] = { };
		if MI2_Debug > 0 then chattext( "M2DBG: creating new DB record for ["..index.."]" ); end
	end  

	if  playerName and MobInfoDB[index][playerName] == nil  then
		MobInfoDB[index][playerName] = { };
	end

	return mobIndex;
end  -- of MI2_MobDbCheck()


-----------------------------------------------------------------------------
-- MI2_RecordDamage()
--
-- record damage value for a mob
-----------------------------------------------------------------------------
local function MI2_RecordDamage( index, creatureName, damage )
	local dl, du, dps;

	-- check: mob must be current target, damage value must be > 0
	if creatureName ~= MI2_Target.name or damage < 1 then
		return;
	end

	-- ensure that Mob is in DB (create new entry if new Mob)
	MI2_MobDbCheck( index, lPlayer );

	-- get current values for dl (damage lower) and du (damage upper)
	dl, du, dps = MI2_GetDamageData( index, lPlayer );

	if MI2_Debug > 1 then chattext( "M2DBG: damage reported: mob=["..index.."], dmg="..damage..", dl="..dl..", du="..du ); end

	-- if mob already has recorded dmg check and set min/max
	if dl == 0 then
		dl, du = damage, damage;
	elseif damage < dl then
		if MI2_Debug > 0 then chattext( "M2DBG: recording new MIN dmg "..damage.." for ["..index.."] (old="..dl..")" ); end
		dl = damage;
	elseif damage > du then
		if MI2_Debug > 0 then chattext( "M2DBG: recording new MAX dmg "..damage.." for ["..index.."] (old="..du..")" ); end
		du = damage;
	end
	
	-- store modified damage data in database
	MobInfoDB[index][lPlayer].dd = dl.."/"..du.."/"..dps;
end  -- of MI2_RecordDamage()


-----------------------------------------------------------------------------
-- MI2_RecordDps()
--
-- record a new dps (damage per second) value for a given mob
-- dps is recorded as a player specific value, it gets calculated from
-- given values for time and damage done within that time
-----------------------------------------------------------------------------
function MI2_RecordDps( index, deltaTime, damage  )
	local dl, du, dps = 0,0,0;

	-- ensure that Mob is in DB (create new entry if new Mob)
	MI2_MobDbCheck( index, lPlayer );

	-- only store dps for fights longer then 4 seconds
	if deltaTime > 4 then
		-- get existing dl, du, dps values
		local dl, du, dps = MI2_GetDamageData( index, lPlayer );
		if dps == 0 then
			dps = damage / deltaTime;
		end

		-- calculate DPS value
		local delta = MI2_Target.FightEndTime - MI2_Target.FightStartTime;
		local newDps = floor( ((2.0 * dps) + (damage / deltaTime)) / 3.0 );

		-- update the dd (damage data) entry for this mob
		MobInfoDB[index][lPlayer].dd = dl.."/"..du.."/"..newDps;
		if MI2_Debug > 0 then chattext( "M2DBG: recording new dps: idx="..index..", old dps="..dps..", new ddd="..MobInfoDB[index][lPlayer].dd); end
	else
		if MI2_Debug > 0 then chattext( "M2DBG: dps not recorded: idx="..index..", dt="..deltaTime..", dmg="..damage ); end
	end
end  -- of MI2_RecordDps()


-----------------------------------------------------------------------------
-- MI2_RecordKill()
--
-- record one kill for the given mob / player
-- problem : WoW will often generate 2 kill messages per Mob : one with
-- XP and one without XP. This dow not not happen every time, though.
-----------------------------------------------------------------------------
local function MI2_RecordKill( index, playerName, xp )
	-- ensure that Mob is in DB (create new entry if new Mob)
	MI2_MobDbCheck( index, playerName );

	-- record the XP value if option set
	if  xp > 0  and  (MobInfoConfig.SaveAllValues == 1 or MobInfoConfig.ShowXp == 1)  then
		if  MobInfoDB[index].xp ~= xp then
			if MI2_Debug > 0 then chattext( "M2DBG: recording XP "..xp.." for mob ["..index.."]" ); end
		end
		MobInfoDB[index].xp = xp;
	end

	-- exit at this point if recording of kills is disabled
	if  MobInfoConfig.SaveAllValues ~= 1  and  MobInfoConfig.ShowKills == 0  then
		return;
	end

	-- only count kill if it is a new kill
	if  xp <= 0  or index ~= lLastRecordedKill  then
		-- if kill count exists increment it by 1, if not initialise it to 1
		MobInfoDB[index][playerName].kl = (MobInfoDB[index][playerName].kl or 0) + 1;
		if MI2_Debug > 0 then chattext( "M2DBG: recording mob kill for ["..index.."]" ); end
	end
end  -- of MI2_RecordKill()


-----------------------------------------------------------------------------
-- MobInfoDbSet()
--
-- Add given values for the selected mob
-----------------------------------------------------------------------------
local function MobInfoDbSet(index, copper, itemvalue, loots, rareIndex, clothdrop, emptyloots )

	-- ensure that Mob is in DB (create new entry if new Mob)
	MI2_MobDbCheck( index, lPlayer );

	if copper and copper ~= 0 then
		if (MobInfoConfig.SaveAllValues == 1) or (MobInfoConfig.ShowTotal == 1) or (MobInfoConfig.ShowCoin == 1) then
			local c = GetMobCopper(index);
			MobInfoDB[index].cp = c + copper;
		end
	end

	if itemvalue and itemvalue  > 0 then
		if (MobInfoConfig.SaveAllValues == 1) or (MobInfoConfig.ShowTotal == 1) or (MobInfoConfig.ShowIV == 1) then
			local iv = GetMobIV(index);
			if (iv + itemvalue) > 0 then
				MobInfoDB[index].iv = iv + itemvalue; 
			else
				MobInfoDB[index].iv = 0;
			end        
		end
	end

	if loots and loots > 0 then
		local l = GetMobLoots(index); 
		MobInfoDB[index].lt = l + loots;   
	end

	if  emptyloots  and emptyloots > 0  then
		if MobInfoConfig.SaveAllValues == 1 or MobInfoConfig.ShowEmpty == 1 then
			if not MobInfoDB[index].el then
				MobInfoDB[index].el = 0;
			end
			MobInfoDB[index].el = MobInfoDB[index].el + 1;
		end
	end

	if rareIndex then
		if (MobInfoConfig.SaveAllValues == 1) or (MobInfoConfig.ShowQuality == 1) then
			local rn = GetMobRarity(index, rareIndex);
			if rareIndex == 1 then 
				MobInfoDB[index].r0 = rn + 1;
			elseif rareIndex == 2 then
				MobInfoDB[index].r1 = rn + 1;
			elseif rareIndex == 3 then
				MobInfoDB[index].r2 = rn + 1;
			elseif rareIndex == 4 then
				MobInfoDB[index].r3 = rn + 1;
			elseif rareIndex == 5 then
				MobInfoDB[index].r4 = rn + 1;
			end
		end
	end

	if clothdrop ~= nil and clothdrop ~= 0 then
		if (MobInfoConfig.SaveAllValues == 1) or (MobInfoConfig.ShowCloth == 1) then
			local cd = GetMobCloth(index);
			MobInfoDB[index].cc = cd + clothdrop;
		end
	end

	-- clean up from previous versions
	if MobInfoDB[index].class then
		MobInfoDB[index].class = nil;
	end    
end  -- of MobInfoDbSet()


-----------------------------------------------------------------------------
-- MI2_CleanupMobData()
--
-- Clean up the Mob database entry. This removes any data that should not
-- be saved in the database (due to option settings) and updates old
-- databse entries from very old MobInfo evrsions to new ones.
-----------------------------------------------------------------------------
function MI2_CleanupMobData( index )
	-- nothing to be done if mob is not in MobInfo database
	if not MobInfoDB[index] then
		return;
	end

	-- delete any unused keys if the user has save all values off  
	if MobInfoConfig.SaveAllValues == 0 then
		if MobInfoConfig.ShowNo2lev == 0 then -- not saved in sv
			if MobInfoConfig.ShowXp == 0 then -- can only remove it if show no2level is off also
				if MobInfoDB[index].xp then MobInfoDB[index].xp = nil end
			end
		end

		if MobInfoConfig.ShowTotal == 0 then -- not saved in sv
			if MobInfoConfig.ShowCoin == 0 then -- can only remove it if show total is off also
				if MobInfoDB[index].cp then MobInfoDB[index].cp = nil end
			end
			if MobInfoConfig.ShowIV == 0 then -- can only remove if show total is also off
				if MobInfoDB[index].iv then MobInfoDB[index].iv = nil end
			end
		end

		if MobInfoConfig.ShowQuality == 0 then
			if MobInfoDB[index].r0 then MobInfoDB[index].r0 = nil end
			if MobInfoDB[index].r1 then MobInfoDB[index].r1 = nil end
			if MobInfoDB[index].r2 then MobInfoDB[index].r2 = nil end
			if MobInfoDB[index].r3 then MobInfoDB[index].r3 = nil end
			if MobInfoDB[index].r4 then MobInfoDB[index].r4 = nil end
		end

		if MobInfoConfig.ShowCloth == 0 then
			if MobInfoDB[index].cc then MobInfoDB[index].cc = nil end
		end

		if MobInfoConfig.ShowEmpty == 0 then
			if MobInfoDB[index].el then MobInfoDB[index].el = nil end
		end

		-- cleanup player specific data: damage data and kills
		if MobInfoDB[index][lPlayer] then
			if MobInfoConfig.ShowDamage == 0 then
				if MobInfoDB[index][lPlayer].dd then MobInfoDB[index][lPlayer].dd = nil end
			end
			if MobInfoConfig.ShowKills == 0 then
				if MobInfoDB[index][lPlayer].kl then MobInfoDB[index][lPlayer].kl = nil end 
			end
			if not MobInfoDB[index][lPlayer].dd and not MobInfoDB[index][lPlayer].kl and not MobInfoDB[index][lPlayer].dl then
				MobInfoDB[index][lPlayer] = nil;
			end
		end

		-- To turn off loots all the loot stuff must be turned off
		if  MobInfoConfig.ShowTotal == 0 and
			MobInfoConfig.ShowCoin == 0 and
			MobInfoConfig.ShowIV == 0 and
			MobInfoConfig.ShowCloth == 0 and
			MobInfoConfig.ShowLoots == 0 then
				if MobInfoDB[index].lt then MobInfoDB[index].lt = nil end
		end      
	end
end


-----------------------------------------------------------------------------
-- MI2_IndexComponents()
--
-- Return the component parts of a mob index: mob name, mob level
-----------------------------------------------------------------------------
function MI2_GetIndexComponents( mobIndex )
	local a, b, mobName, mobLevel = string.find(mobIndex, "(%w+):(%d+)$");
	mobLevel = tonumber(mobLevel);
	return mobName, mobLevel;
end  -- MI2_IndexComponents()


-----------------------------------------------------------------------------
-- MI2_GetMobHealthStr()
--
-- Returns the mobhealth in the form of xx/xx from the mobdb formed by
-- MobHealth mod Pulled from Telo's MobHealth
-----------------------------------------------------------------------------
local function MI2_GetMobHealthStr( index, healthPercent )
  if not UnitIsFriend("player","mouseover") then    
    local pointsPerPct = MobHealth_PPP( index );
    local currentPct = healthPercent;
	  if( pointsPerPct > 0 ) then	
		  if( currentPct ) then
			  return string.format("%d/%d", (currentPct * pointsPerPct) + 0.5, (100 * pointsPerPct) + 0.5);
		  else
			  return string.format("???/%d", (100 * pointsPerPct) + 0.5);
		  end
      end
  end
  return nil;
end  -- of MI2_GetMobHealthStr()


-----------------------------------------------------------------------------
-- copper2text()
--
-- Turns a full copper amount to a readable string, eg. 10340 = 1g 3s 40c
-----------------------------------------------------------------------------
function copper2text(copper)
	local g,s,c;
		
	g = floor(copper / COPPER_PER_GOLD);
	s = floor(copper / COPPER_PER_SILVER) - g * SILVER_PER_GOLD;
	c = copper - g * COPPER_PER_GOLD - s * COPPER_PER_SILVER;

	if g > 0 then  
  		return mifontWhite..g..mifontYellow..'g '..mifontWhite..s ..mifontSubWhite..'s '..mifontWhite..c..mifontGold..'c';
	end  

	if s > 0 then  
  		return mifontWhite..s ..mifontSubWhite..'s '..mifontWhite..c..mifontGold..'c';
	end  

	return mifontWhite..c..mifontGold..'c';
end


-----------------------------------------------------------------------------
-- lootName2Copper()
--
-- Turns a lootname like 1 Gold 3 Silver 40 Copper to total copper 10340
-----------------------------------------------------------------------------
function lootName2Copper(item)
  local i = 0;
  local g,s,c = 0;
  
  i = string.find(item, MI_TXT_GOLD  );
  if i then
    g = string.sub(item,0,i-1);
    item = string.sub(item,i+5,string.len(item));
  end
  i = 0;
  i = string.find(item, MI_TXT_SILVER );
  if i then
    s = string.sub(item,0,i-1);
    item = string.sub(item,i+7,string.len(item));
  end
  i = 0;
  i = string.find(item, MI_TXT_COPPER );
  if i then
    c = string.sub(item,0,i-1);
  end

  local money = 0;
  if c then
		money = money + c;
	end
	if s then
		money = money + (s * COPPER_PER_SILVER);
	end
	if g then
		money = money + (g * COPPER_PER_GOLD);
	end

  return money;
end  -- of lootName2Copper()


-----------------------------------------------------------------------------
-- MI2_FindItemValue()
--
-- Find the item value in either the Auctioneer database or in out own copy
-- of the Auctioneer item value database or by asking KC_Items
-----------------------------------------------------------------------------
local function MI2_FindItemValue( itemID, link )
  -- check if KC_Items is available and knows the price
  if  KC_SellValue  then
    local price = KC_SellValue:GetPrice( KC_Items:GetCode(link) );
    if  price  and price > 0  then
      return price;
    end
  end

  -- check if Auctioneer is installed and knows the price
  if  Auctioneer_GetVendorBuyPrice  then
    local price = Auctioneer_GetVendorSellPrice(itemID);
    if  price  then
      return price;
    end
  end

  -- check if built-in copy of the Auctioneer base prices knows the item price
  if  MI2_BasePrices  and  MI2_BasePrices[itemID]  then
    return tonumber( MI2_BasePrices[itemID] );
  end

  return 0;  
end  -- of MI2_FindItemValue()


-----------------------------------------------------------------------------
-- GetLootId()
--
-- get loot ID code for given loot slot number, also return link object
-----------------------------------------------------------------------------
local function GetLootId( slot )

  local idNumber = 0;

  local link = GetLootSlotLink( slot );
  if link then
    local d1, d2, idCode = string.find(link, "|Hitem:(%d+):(%d+):(%d+):");
    idNumber = tonumber( idCode or 0 );
  end
  
  return idNumber, link;
end  -- of GetLootId()
  

-----------------------------------------------------------------------------
-- MI2_GetCorpseId()
--
-- create a (hopefully) unique corpse ID out of the loot items found in 
-- the corpse loot window
-----------------------------------------------------------------------------
local function MI2_GetCorpseId( index )
  local corpseId = index;
  local numItems = 0;
  
  for slot = 1, GetNumLootItems(), 1 do
    -- obtain loot slot data from WoW
    local texture, item, quantity, quality = GetLootSlotInfo( slot );
    if  item ~= ""  then
      corpseId = corpseId.."_"..item;
      numItems = numItems + 1;
    end
  end
  
  return corpseId..numItems;
end -- of CheckForLastLoot()
  

-----------------------------------------------------------------------------
-- CheckForLastLoot()
--
-- check if the current open corpse is identical to the last opened corpse
-- return true if both are identical
-----------------------------------------------------------------------------
local function CheckForLastLoot( index )
  -- create corpse ID
  local corpseId = MI2_GetCorpseId( index );
  
  if MI2_Debug > 1 then chattext( "M2DBG: corpseId=["..corpseId.."], last1=["..lLastLoot1.."], last2=["..lLastLoot2.."]" ); end
  
  -- check for identical last loot and reset last loot buffer
  local isLastLoot =  corpseId == lLastLoot1  or  corpseId == lLastLoot2;
  
  if not isLastLoot then
    if lLastLootNum == 1 then
      lLastLoot1 = corpseId;
      lLastLootNum = 2;
    else
      lLastLoot2 = corpseId;
      lLastLootNum = 1;
    end
  end
  
  return isLastLoot;
end -- of CheckForLastLoot()
  

-----------------------------------------------------------------------------
-- MI2_CorpseOpened()
--
-- a corpse has been opened: increment loot counter and process all loot items
-----------------------------------------------------------------------------
local function MI2_CorpseOpened( index )

  local skin = 0; -- to detect whether its a skinning loot window
  
  if MI2_Debug > 0 then chattext( "M2DBG: corpse opened for : ["..index.."], numItems="..GetNumLootItems() ); end
 
  -- compare this loot to the last recorded Mob loot
  -- dont process this loot if its identical to last loot stored
  if CheckForLastLoot(index) then
    if MI2_Debug > 0 then chattext( "M2DBG: loot REOPEN detected" ); end
    return 0;
  end
  
  for slot = 1, GetNumLootItems(), 1 do
    local cloth  = 0;
    local iv     = 0;
    local copper = 0;

    -- obtain loot slot data from WoW
    local link = GetLootSlotLink( slot );
    local texture, item, quantity, quality = GetLootSlotInfo( slot );
    
    -- if we find skinning loot we do not increment the "looted" counter
    if  miSkinLoot[item]  and  slot == 1  then
      skin = 1;
    end

    -- process custom tracking items
    if MobInfoConfig.CustomTracks[item] then
      MobInfoDbSetCustom(index,item,quantity);
    end;

    -- check for cloth loot items
    if miClothLoot[item] then
      cloth = 1;
    end
  
    -- calculate value of coin loot
    if LootSlotIsCoin(slot) then
      copper = lootName2Copper(item);
      quality = -1;
    end
    
    -- calculate value of item loot, dont count qulity for items without value
    local itemID, link = GetLootId( slot );
    if LootSlotIsItem(slot) then
      iv = MI2_FindItemValue( itemID, link );
      if  iv < 1  then
        quality = -1;
        if MI2_Debug > 1 then chattext( "M2DBG: quality count skipped for ["..item.."]" ); end
      end
    end
    
    -- add loot item data to MobInfoDB
    MobInfoDbSet( index, copper, iv, 0, quality+1, cloth );
    if MI2_Debug > 1 then chattext( "M2DBG: Loot: slot="..slot..", name=["..item.."], id=["..itemID.."], val=["..iv.."], q=["..(quality+1).."]" ); end
  end  -- of for loop
  
  -- add loot count to mob if its not a skinning loot
  if  skin == 0 then
    MobInfoDbSet(index,0,0,1);  -- add a loot to the mob
  end
end -- of MI2_CorpseOpened()  


-----------------------------------------------------------------------------
-- MI2_EventLootOpened()
--
-- WoW event notification that loot frame has been opened
-----------------------------------------------------------------------------
local function MI2_EventLootOpened( )
  local index = MI2_Target.mobIndex;

  if index and GetNumLootItems() > 0 then
    -- set lLastRecordedKill to prevent counting this Mob as a kill without XP while looting
    lLastRecordedKill = index;
    lLastLooted = index;

    -- process loot when opening a corpse we killed
    if UnitIsDead("target") and UnitCreatureType("target") 
       and not UnitIsPlayer("target") 
       and not UnitIsFriend("player","target")
    then 
      MI2_CorpseOpened( index );
      return 0;
    end
  end
end  -- of MI2_EventLootOpened()


-----------------------------------------------------------------------------
-- MI2_EventLootClosed()
--
-- WoW event notification that loot frame has been closed
-- This event is used to detect empty loots. Epty loots are loots where
-- no previous"LOOT_OPENED" has been recorded
-----------------------------------------------------------------------------
local function MI2_EventLootClosed( )
  local index = MI2_Target.mobIndex;

  if index then
    if MI2_Debug > 1 then chattext( "M2DBG: loot closed : lastLoot=["..index.."], numItems="..GetNumLootItems() ); end
  
    -- reset lLastRecordedKill when loot window closes as there is no more danger of wrong kill counts
    lLastRecordedKill = "";

    if  lLastLooted == ""  then
      MobInfoDbSet( index,0,0,1,0,0,1 );  -- add a loot and emptyloot to the mob
      if MI2_Debug > 0 then chattext( "M2DBG: recording empty loot for ["..index.."]" ); end
    end
  end
  
  lLastLooted = "";
end  -- of MI2_EventLootOpened()


-----------------------------------------------------------------------------
-- MI2_EventLootSlotCleared()
--
-- WoW event notification that one loot item has been looted
-----------------------------------------------------------------------------
local function MI2_EventLootSlotCleared( )
  local corpseId ="-";

  -- create new corpse ID
  if  MI2_Target.mobIndex  then
    corpseId = MI2_GetCorpseId( MI2_Target.mobIndex );
  end
  
  if MI2_Debug > 2 then chattext( "M2DBG: looting: id=["..corpseId.."], last1=["..lLastLoot1.."], last2=["..lLastLoot2.."]" ); end
  
  if lLastLootNum == 1 then
    lLastLoot2 = corpseId;
  else
    lLastLoot1 = corpseId;
  end
end  -- of MI2_EventLootSlotCleared


-----------------------------------------------------------------------------
-- MI2_CreateNormalTooltip()
--
-- add all collected mob data to the game tooltip, data is only added if
-- corresponding "Show" flag is set
-----------------------------------------------------------------------------
local function MI2_CreateNormalTooltip( mobData, mobIndex )
	local copperAvg, itemValueAvg;
	local addEmptyLine = 0;
	
	-- do not add anything to the tooltip for players
	if  UnitIsPlayer("mouseover")  then  return;  end

	if mobData.class and MobInfoConfig.ShowClass == 1 then
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_CLASS, mifontWhite..mobData.class );
	end

	if mobData.healthCur and MobInfoConfig.ShowHealth == 1 then
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_HEALTH, mifontWhite..mobData.healthCur.." / "..mobData.healthMax );
		lMiHealthLine = GameTooltip:NumLines();
	end

	if mobData.manaMax and mobData.manaMax > 0 and MobInfoConfig.ShowMana == 1 then
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_MANA, mifontWhite..mobData.manaCur.." / "..mobData.manaMax );
		lMiManaLine = GameTooltip:NumLines();
	end

	-- exit right here if mob does not exist in database
	if not mobData.color then
		return;
	end

	local mobGivesXp = not (mobData.color.r == 0.5  and  mobData.color.g == 0.5  and  mobData.color.b == 0.5);
	if mobGivesXp and mobData.xp then
		if MobInfoConfig.ShowXp == 1 then
			GameTooltip:AddDoubleLine( mifontGold..MI_TXT_XP, mifontWhite..mobData.xp );
		end 
		if MobInfoConfig.ShowNo2lev == 1 then
			GameTooltip:AddDoubleLine( mifontGold..MI_TXT_TO_LEVEL, mifontWhite..mobData.mob2Level );
		end 
	end

	if (mobData.minDamage or mobData.dps) and MobInfoConfig.ShowDamage == 1 then 
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_DAMAGE, mifontWhite..mobData.minDamage.."-"..mobData.maxDamage.."  ["..mobData.dps.."]" );
	end

	addEmptyLine = MobInfoConfig.ShowBlankLines;

	if  MobInfoConfig.CombinedMode == 1  and  MobInfoConfig.ShowCombined == 1  then
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddLine( mifontGray.."["..MI_TXT_COMBINED..mobData.combinedStr.."]" );
	end

	if mobData.kills and MobInfoConfig.ShowKills == 1 then
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_KILLS, mifontWhite..mobData.kills );
	end          

	if  mobData.loots  and  MobInfoConfig.ShowLoots == 1  then
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_TIMES_LOOTED, mifontWhite..mobData.loots );
	end

	if  mobData.emptyLoots  and  MobInfoConfig.ShowEmpty == 1  then
		local emptyLootsStr = mifontWhite..mobData.emptyLoots;
		if  mobData.loots  then
			emptyLootsStr = emptyLootsStr.." ("..ceil((mobData.emptyLoots/mobData.loots)*100).."%) "
		end
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_EMPTY_LOOTS, emptyLootsStr );
	end

	addEmptyLine = MobInfoConfig.ShowBlankLines;

	-- Add custom values
	for key, value in MobInfoConfig.CustomTracks do
		local c = GetMobCustom( mobIndex, key );
		if c > 0 and mobData.loots > 0 then
			GameTooltip:AddDoubleLine(mifontYellow..key,mifontWhite..c.." ("..ceil((c/mobData.loots)*100).."%) ");
		end
	end

	if  mobData.qualityStr ~= ""  and  MobInfoConfig.ShowQuality == 1  then
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddDoubleLine(mifontGold..MI_TXT_QUALITY, mobData.qualityStr);
	end

	if mobData.clothCount and MobInfoConfig.ShowCloth == 1 then
		local clothStr = mifontWhite..mobData.clothCount;
		if mobData.loots then
			clothStr = clothStr.." ("..ceil((mobData.clothCount/mobData.loots)*100).."%) "
		end
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddDoubleLine( mifontGold..MI_TXT_CLOTH_DROP, clothStr );
	end

	if mobData.copper and mobData.loots then
		copperAvg = ceil( mobData.copper / mobData.loots );
		if MobInfoConfig.ShowCoin == 1 then
			if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
			GameTooltip:AddDoubleLine(mifontGold..MI_TXT_COIN_DROP,mifontWhite..copper2text(copperAvg));
		end
	end

	if mobData.itemValue and mobData.loots then
		itemValueAvg = ceil( mobData.itemValue / mobData.loots );
		if MobInfoConfig.ShowIV == 1 then
			if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
			GameTooltip:AddDoubleLine(mifontGold..MI_TEXT_ITEM_VALUE,mifontWhite..copper2text(itemValueAvg));
		end
	end

	local totalValue = (copperAvg or 0) + (itemValueAvg or 0);
	if totalValue > 0 and MobInfoConfig.ShowTotal == 1 then
		if addEmptyLine == 1 then GameTooltip:AddLine("\n"); addEmptyLine = 0; end
		GameTooltip:AddDoubleLine(mifontGold..MI_TXT_MOB_VALUE,mifontWhite..copper2text(totalValue));
	end

  -----------------------------------------------------------------------
  -- debugging code : append actual database contents to end of tooltip
  -- enabled by setting local vairable "MI2_Debug"
  if MI2_Debug > 1 then
  GameTooltip:AddLine("----------------  D e b u g   I n f o  ----------------");
  GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."index",mobIndex);
  if MobInfoDB[mobIndex] and lPlayer then
    if MobInfoDB[mobIndex].xp then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."experience(xp)",mifontWhite..MobInfoDB[mobIndex].xp); end;
    if MobInfoDB[mobIndex].cp then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."copper(cp)",mifontWhite..MobInfoDB[mobIndex].cp); end;
    if MobInfoDB[mobIndex].iv then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."itemvalue(iv)",mifontWhite..MobInfoDB[mobIndex].iv); end;
    if MobInfoDB[mobIndex].lt then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."loot(lt)",mifontWhite..MobInfoDB[mobIndex].lt); end;
    if MobInfoDB[mobIndex].cc then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."clothcount(cc)",mifontWhite..MobInfoDB[mobIndex].cc); end;
    if MobInfoDB[mobIndex].el then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."empty loots(el)",mifontWhite..MobInfoDB[mobIndex].el); end;
    GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."quality(rx)",mifontWhite.."("..(MobInfoDB[mobIndex].r0 or 0)..","..(MobInfoDB[mobIndex].r1 or 0)..","..(MobInfoDB[mobIndex].r2 or 0)..","..(MobInfoDB[mobIndex].r3 or 0)..","..(MobInfoDB[mobIndex].r4 or 0)..")");
    if MobInfoDB[mobIndex].bi then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."basic info(bi)",mifontWhite..MobInfoDB[mobIndex].bi); end;
    if MobInfoDB[mobIndex][lPlayer] then
    if MobInfoDB[mobIndex][lPlayer].kl then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."kills(kl)",mifontWhite..MobInfoDB[mobIndex][lPlayer].kl); end;
    if MobInfoDB[mobIndex][lPlayer].dl then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."damage range min(dl)",mifontWhite..MobInfoDB[mobIndex][lPlayer].dl); end;
    if MobInfoDB[mobIndex][lPlayer].du then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."damage range max(du)",mifontWhite..MobInfoDB[mobIndex][lPlayer].du); end;
    if MobInfoDB[mobIndex][lPlayer].dd then GameTooltip:AddDoubleLine("[DBG] "..mifontGold.."damage data(dd)",mifontWhite..MobInfoDB[mobIndex][lPlayer].dd); end;
    end
  end end
  -- end of debugging code
  -----------------------------------------------------------------------
end  -- of MI2_CreateNormalTooltip()


-----------------------------------------------------------------------------
-- MI2_CreateCompactTooltip()
--
-- add all collected mob data to the game tooltip, data is only added if
-- corresponding "Show" flag is set
-----------------------------------------------------------------------------
local function MI2_CreateCompactTooltip( mobData, mobIndex )
	-- do not add anything to the tooltip for players
	if  UnitIsPlayer("mouseover")  then  return;  end
	
	local firstLine = GameTooltip:NumLines() + 1;

	if (mobData.healthCur or mobData.manaMax and mobData.manaMax > 0)  
			and (MobInfoConfig.ShowHealth == 1 or MobInfoConfig.ShowMana == 1) then
		GameTooltip:AddDoubleLine( mifontGold.."HP    "..mifontWhite..(mobData.healthCur or 0).."/"..(mobData.healthMax or 0), mifontWhite..(mobData.manaCur or 0).."/"..(mobData.manaMax or 0)..mifontGold.." Mana" );
	end

	-- exit right here if mob does not exist in database
	if not mobData.color then
		return;
	end

	local mobGivesXp = not (mobData.color.r == 0.5  and  mobData.color.g == 0.5  and  mobData.color.b == 0.5);
	if mobGivesXp and mobData.xp and (MobInfoConfig.ShowXp == 1 or MobInfoConfig.ShowNo2lev == 1) then
		GameTooltip:AddDoubleLine( mifontGold.."XP    "..mifontWhite..mobData.xp, mifontWhite..mobData.mob2Level..mifontGold.." KtL    " );
	end

	if (mobData.minDamage or mobData.dps) and MobInfoConfig.ShowDamage == 1 then
		GameTooltip:AddDoubleLine( mifontGold.."Dmg "..mifontWhite..(mobData.minDamage or 0).."-"..(mobData.maxDamage or 0), mifontWhite..(mobData.dps or 0)..mifontGold.." Dps   " );
	end

	if  MobInfoConfig.CombinedMode == 1  and  MobInfoConfig.ShowCombined == 1  then
		GameTooltip:AddLine( mifontGray.."["..MI_TXT_COMBINED..mobData.combinedStr.."]" );
	end

	if (mobData.kills or mobData.loots) and (MobInfoConfig.ShowKills == 1 or MobInfoConfig.ShowLoots == 1)  then
		GameTooltip:AddDoubleLine( mifontGold.."Kills  "..mifontWhite..(mobData.kills or 0), mifontWhite..(mobData.loots or 0)..mifontGold.." Loots" );
	end          

	if  (mobData.emptyLoots or mobData.clothCount) and (MobInfoConfig.ShowCloth == 1 or MobInfoConfig.ShowEmpty == 1)  then
		local emptyLootsStr = mifontWhite..(mobData.emptyLoots or 0);
		if  mobData.loots  then
			emptyLootsStr = emptyLootsStr.." ("..ceil(((mobData.emptyLoots or 0)/mobData.loots)*100).."%) "
		end
		local clothStr = mifontWhite..(mobData.clothCount or 0);
		if mobData.loots then
			clothStr = clothStr.." ("..ceil(((mobData.clothCount or 0)/mobData.loots)*100).."%) "
		end
		GameTooltip:AddDoubleLine( mifontGold.."CL     "..mifontWhite..clothStr, mifontWhite..emptyLootsStr..mifontGold.." EL      " );
	end

	if (mobData.copper or mobData.itemValue) and mobData.loots and MobInfoConfig.ShowTotal == 1 then
		local copperAvg = ceil( (mobData.copper or 0) / mobData.loots );
		local itemValueAvg = ceil( (mobData.itemValue or 0) / mobData.loots );
		local totalValue = copperAvg + itemValueAvg;
		if totalValue > 0 then
			GameTooltip:AddDoubleLine( mifontGold.."Val    "..mifontWhite..copper2text(totalValue), mifontWhite..copper2text(copperAvg)..mifontGold.." Coins" );
		end
	end

	if  mobData.qualityStr ~= ""  and  MobInfoConfig.ShowQuality == 1  then
		GameTooltip:AddLine( mifontGold.."Q      "..mifontWhite..mobData.qualityStr );
	end

	local lastLine = GameTooltip:NumLines();
	local line;
	local maxWidth = 0
	
	for i = firstLine, lastLine do
		line = getglobal( "GameTooltipTextRight"..i );
		if line:GetStringWidth() > maxWidth then
			maxWidth = line:GetStringWidth();
		end
	end

--	for i = firstLine, lastLine do
--		line = getglobal( "GameTooltipTextRight"..i );
--		line:SetWidth( maxWidth + 4 );
--		line:SetJustifyH("LEFT");
--	end
end  -- of MI2_CreateCompactTooltip()


-----------------------------------------------------------------------------
-- MI2_CreateQualityString()
--
-- add the data for two mobs,
-- the data of the second mob (mobData2) is added to the data of the first
-- mob (mobData1)
-----------------------------------------------------------------------------
local function MI2_AddTwoMobs( mobData1, mobData2 )
	mobData1.loots = (mobData1.loots or 0) + (mobData2.loots or 0);
	mobData1.kills = (mobData1.kills or 0) + (mobData2.kills or 0);
	mobData1.emptyLoots = (mobData1.emptyLoots or 0) + (mobData2.emptyLoots or 0);
	mobData1.clothCount = (mobData1.clothCount or 0) + (mobData2.clothCount or 0);
	mobData1.copper = (mobData1.copper or 0) + (mobData2.copper or 0);
	mobData1.itemValue = (mobData1.itemValue or 0) + (mobData2.itemValue or 0);
	mobData1.r1 = (mobData1.r1 or 0) + (mobData2.r1 or 0);
	mobData1.r2 = (mobData1.r2 or 0) + (mobData2.r2 or 0);
	mobData1.r3 = (mobData1.r3 or 0) + (mobData2.r3 or 0);
	mobData1.r4 = (mobData1.r4 or 0) + (mobData2.r4 or 0);
	mobData1.r5 = (mobData1.r5 or 0) + (mobData2.r5 or 0);
	
	if mobData1.loots == 0 then mobData1.loots = nil; end
	if mobData1.kills == 0 then mobData1.loots = nil; end
	if mobData1.emptyLoots == 0 then mobData1.emptyLoots = nil; end
	if mobData1.clothCount == 0 then mobData1.clothCount = nil; end
	if mobData1.copper == 0 then mobData1.copper = nil; end
	if mobData1.itemValue == 0 then mobData1.itemValue = nil; end
	if mobData1.r1 == 0 then mobData1.r1 = nil; end
	if mobData1.r2 == 0 then mobData1.r2 = nil; end
	if mobData1.r3 == 0 then mobData1.r3 = nil; end
	if mobData1.r4 == 0 then mobData1.r4 = nil; end
	if mobData1.r5 == 0 then mobData1.r5 = nil; end
end  -- MI2_AddTwoMobs


-----------------------------------------------------------------------------
-- MI2_BuildQualityString()
--
-- build mob data structure required for displaying game tooltip
-----------------------------------------------------------------------------
local function MI2_BuildQualityString( mobData )
	local rt = mobData.loots or 1; 
 
	mobData.qualityStr = "";

	if  mobData.r1  then
		mobData.qualityStr = mobData.qualityStr ..mifontGray..mobData.r1.."("..ceil((mobData.r1/rt)*100).."%) ";
	end
	if  mobData.r2  then
		mobData.qualityStr = mobData.qualityStr ..mifontWhite..mobData.r2.."("..ceil((mobData.r2/rt)*100).."%) ";
	end
	if  mobData.r3  then
		mobData.qualityStr = mobData.qualityStr ..mifontGreen..mobData.r3.."("..ceil((mobData.r3/rt)*100).."%) ";
	end
	if  mobData.r4  then
		mobData.qualityStr = mobData.qualityStr ..mifontBlue..mobData.r4.."("..ceil((mobData.r4/rt)*100).."%) ";
	end
	if  mobData.r5  then
		mobData.qualityStr = mobData.qualityStr ..mifontMageta..mobData.r5.."("..ceil((mobData.r5/rt)*100).."%) ";
	end
end  -- MI2_CreateQualityString


-----------------------------------------------------------------------------
-- MI2_BuildMobInfoTooltip()
--
-- create game tooltip contents
-----------------------------------------------------------------------------
local function MI2_BuildMobInfoTooltip( mobName, mobLevel )
	-- get mob data for hovered mob
	local mobData = MI2_GetMobData( mobName, mobLevel, "mouseover" );
	mobData.combinedStr = "";

	lMiMouseoverIndex = mobName..":"..mobLevel;

	-- store mob classification in database (if not "normal")
	if mobData.color then
		local mobType  = UnitClassification( "mouseover" );
		if mobType ~= "normal" then
			if mobType == "rare" or mobType == "elite" then
				MobInfoDB[lMiMouseoverIndex].mt = "2";
			else
				MobInfoDB[lMiMouseoverIndex].mt = "3";
			end
		end
	end

	-- handle combined Mob mode : try to find the other Mobs with same
	-- name but differing level, add their data to the tooltip data
	if  MobInfoConfig.CombinedMode == 1 and mobLevel > 0 then
		if mobLevel < 4 then mobLevel = 4 end
		for levelToCombine = mobLevel-3, mobLevel+3, 1 do
			if levelToCombine ~= mobLevel  then
				local dataToCombine = MI2_GetMobData( mobName, levelToCombine );
				if dataToCombine.color then
					MI2_AddTwoMobs( mobData, dataToCombine );
					mobData.combinedStr = mobData.combinedStr.." L"..levelToCombine;
				end
			else
				mobData.combinedStr = mobData.combinedStr.." L"..levelToCombine;
			end
		end
	end
  
	-- add the Mob data to the game tooltip
	MI2_BuildQualityString( mobData );
	if MobInfoConfig.CompactMode == 1 then
		MI2_CreateCompactTooltip( mobData, lMiMouseoverIndex );
	else
		MI2_CreateNormalTooltip( mobData, lMiMouseoverIndex );
	end

end  -- of MI2_BuildMobInfoTooltip()


-----------------------------------------------------------------------------
-- MI2_GameTooltip_OnEvent()
--
-- Add new information to the game tip for the selected mob 
-----------------------------------------------------------------------------
function MI2_GameTooltip_OnEvent()
	lMi2GameTooltip_OnEvent_Orig(event);
  
	lMiHealthLine, lMiManaLine = nil, nil;

	if  UnitCreatureType("mouseover") 
      and (UnitIsFriend("player","mouseover") == nil)
      and (MobInfoConfig.DisableMobInfo == 0)
      and (MobInfoConfig.KeypressMode == 0
           or MobInfoConfig.KeypressMode == 1 and IsAltKeyDown())  then
		MI2_BuildMobInfoTooltip( UnitName("mouseover"), UnitLevel("mouseover") );
		GameTooltip:Show();
	end  
end  -- of MI2_GameTooltip_OnEvent()


-----------------------------------------------------------------------------
-- MI2_DpsCalculation()
--
-- Calculate an updated DPS (damage per second) based on the current target
-- data in "MI2_Target" and the new damage value given as parameter.
-----------------------------------------------------------------------------
local function MI2_DpsCalculation( damage, contextInfo )

if not damage then
	chattext( "MI2_ERROR: chat message parse error in "..contextInfo );
	return;
end

	-- calculate and update DPS for target
	if not MI2_Target.FightStartTime then
		MI2_Target.FightStartTime = GetTime() - 0.5;
		MI2_Target.FightEndTime = GetTime();
		MI2_Target.FightDamage = damage;
		lLastRecordedKill = ""; -- reset kill count tracking when a fight starts
	elseif MI2_Target.FightEndTime then
		MI2_Target.FightEndTime = GetTime();
		MI2_Target.FightDamage = MI2_Target.FightDamage + damage;
		if MI2_Debug > 1 then 
			local delta = MI2_Target.FightEndTime - MI2_Target.FightStartTime;
			local dps = MI2_Target.FightDamage / delta;
			chattext( "MI_DBG: combat: dmg="..MI2_Target.FightDamage..", delta="..delta..", dps="..dps); 
		end
	end
end  -- MI2_DpsCalculation()


-----------------------------------------------------------------------------
-- MI2_EventSelfMelee()
--
-- handler for event CHAT_MSG_COMBAT_SELF_HITS
-- handles normal and critical melee damage
-----------------------------------------------------------------------------
local function MI2_EventSelfMelee( )
	local dmgText = arg1;

if MI2_Debug > 1 then chattext( "M2DBG: MI2_EventSelfMelee: chatText=["..arg1.."]" ); end

	-- normal melee damage
	for  creature, damage in string.gfind( dmgText, MI_PARSE_SELF_MELEE ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_MELEE: c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_MELEE" );
		return;
	end

	-- critical melee damage
	for  creature, damage in string.gfind( dmgText, MI_PARSE_SELF_MELEE_CRIT ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_MELEE_CRIT: c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_MELEE_CRIT" );
		return;
	end
end  -- MI2_EventSelfMelee()


-----------------------------------------------------------------------------
-- MI2_EventSelfSpell()
--
-- handler for event "CHAT_MSG_SPELL_SELF_DAMAGE"
-- handles normal and critical spell damage and damage done by bows/guns
-----------------------------------------------------------------------------
local function MI2_EventSelfSpell( )
	local dmgText = arg1;

if MI2_Debug > 1 then chattext( "M2DBG: MI2_EventSelfSpell: chatText=["..arg1.."]" ); end

	-- normal spell damage
	for  spell, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_SPELL ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_SPELL: spell=["..spell.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_SPELL" );
		return;
	end

	-- critical spell damage
	for  spell, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_SPELL_CRIT ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_SPELL_CRIT: spell=["..spell.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_SPELL_CRIT" );
		return;
	end

	-- damage done by bows/guns
	for  spell, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_BOW ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_BOW: wpn=["..spell.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_BOW" );
		return;
	end

if MI2_Debug > 1 then chattext( "M2DBG: MI_PARSE_SELF_SPELL: *NOT PARSED* !!!" ); end

end -- MI2_EventSelfSpell()


-----------------------------------------------------------------------------
-- MI2_EventSelfPet()
--
-- handler for event "CHAT_MSG_COMBAT_PET_HITS" and "CHAT_MSG_SPELL_PET_DAMAGE"
-- handles normal and critical melee/spell damage done by players pet
-----------------------------------------------------------------------------
local function MI2_EventSelfPet( )
	local dmgText = arg1;

if MI2_Debug > 1 then chattext( "M2DBG: MI2_EventSelfPet: chatText=["..arg1.."]" ); end

	-- damage done by players pet
	for  petName, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_PET ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_PET: pet=["..petName.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_PET" );
		return;
	end

	-- critical damage done by players pet
	for  petName, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_PET_CRIT ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_PET_CRIT: pet=["..petName.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_PET_CRIT" );
		return;
	end

	-- damage done by spells of players pet
	for  petName, spell, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_PET_SPELL ) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_PET_SPELL: pet=["..petName.."], spell=["..spell.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_PET_SPELL" );
		return;
	end

	-- critical damage done by spells of players pet
	for  petName, spell, creature, damage in string.gfind( dmgText, MI_PARSE_SELF_PET_SPELL_CRIT) do
		if MI2_Debug > 1 then chattext( "M2DBG: event SELF_PET_SPELL_CRIT: pet=["..petName.."], spell=["..spell.."], c=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_PET_SPELL_CRIT" );
		return;
	end

	--chattext( "M2DBG: unexpected CHAT_MSG_COMBAT_PET_HITS event : ["..arg1.."]" );
end -- MI2_EventSelfPet()


-----------------------------------------------------------------------------
-- MI2_EventSelfSpellPeriodic()
--
-- handler for event "CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE"
-- handles periodic damage done by spells
-----------------------------------------------------------------------------
local function MI2_EventSelfSpellPeriodic( )
	local dmgText = arg1;

if MI2_Debug > 1 then chattext( "M2DBG: MI2_EventSelfSpellPeriodic: chatText=["..arg1.."]" ); end

	-- periodic spell damage
	for  creature, damage, damageType, spell in string.gfind( dmgText, MI_PARSE_SELF_SPELL_PERIODIC ) do
		if MI2_Debug > 1 then chattext( "M2DBG: MI_PARSE_SELF_SPELL_PERIODIC parsed: spell=["..spell.."], cr=["..creature.."], dmg="..damage ); end
		MI2_DpsCalculation( tonumber(damage), "PARSE_SELF_SPELL_PERIODIC" );
		return;
	end

if MI2_Debug > 1 then chattext( "M2DBG: MI_PARSE_SPELL_DMG: *NOT PARSED* !!!" ); end

end -- MI2_EventSelfSpellPeriodic()


-----------------------------------------------------------------------------
-- MI2_EventSpellDamage()
--
-- handle spell damage chat message event : extract damage value
-----------------------------------------------------------------------------
local function MI2_EventSpellDamage( )
	local dmgText = arg1;
	  
if MI2_Debug > 1 then chattext( "M2DBG: MI2_EventSpellDamage (or BOW): chatText=["..arg1.."]" ); end

	-- parse the spell damage chat text, 
	-- for French client order of spell and creature must be swapped
	if MI2_Target.mobIndex and (MobInfoConfig.ShowDamage or MobInfoConfig.SaveAllValues) then
		for  creature, spell, damage in string.gfind( dmgText, MI_PARSE_SPELL_DMG) do
			if MI2_Debug > 1 then chattext( "M2DBG: MI_PARSE_SPELL_DMG  parsed: spell=["..spell.."], cr=["..creature.."], dmg="..damage ); end
			MI2_RecordDamage( MI2_Target.mobIndex, creature, tonumber(damage) );
			return;
		end
		for  creature, spell, damage in string.gfind( dmgText, MI_PARSE_BOW_DMG) do
			if MI2_Debug > 1 then chattext( "M2DBG: MI_PARSE_BOW_DMG  parsed: spell=["..spell.."], cr=["..creature.."], dmg="..damage ); end
			MI2_RecordDamage( MI2_Target.mobIndex, creature, tonumber(damage) );
			return;
		end
	end

if MI2_Debug > 1 then chattext( "M2DBG: MI_PARSE_SPELL_DMG: *NOT PARSED* !!!" ); end

end  -- of MI2_EventSpellDamage(


-----------------------------------------------------------------------------
-- MI2_EventMeleeDamage()
--
-- handle melee damage chat message event : extract damage value
-----------------------------------------------------------------------------
local function MI2_EventMeleeDamage(  )
	local dmgText = arg1;

if MI2_Debug > 1 then chattext( "M2DBG: MI2_EventMeleeDamage: chatText=["..arg1.."]" ); end

	-- parse melee damage chat messsage
	if MI2_Target.mobIndex and (MobInfoConfig.ShowDamage or MobInfoConfig.SaveAllValues) then
		for creature, damage in string.gfind( dmgText, MI_PARSE_COMBAT_DMG ) do
			MI2_RecordDamage( MI2_Target.mobIndex, creature, tonumber(damage) );
		end
	end
end  -- of MI2_EventMeleeDamage()


-----------------------------------------------------------------------------
-- MI2_EventCreatureDiesXP()
--
-- event handler for the chat message telling us that a creature died
-- and gave us XP points
-----------------------------------------------------------------------------
function MI2_EventCreatureDiesXP()
	local idx = MI2_LastTargetIdx;

	if MI2_Debug > 0 then chattext( "M2DBG: mob died with XP: arg1=["..(arg1 or "nil").."], lastTarget=["..(MI2_LastTargetIdx or "nil").."], lastKill=["..(lLastRecordedKill or "nil").."]" ); end

	-- capture XP kills, only count them if they have not already been
	-- counted as non XP kills, otherwise merely log XP
	for creatureName, xp in string.gfind(arg1, MI_MOB_DIES_WITH_XP ) do
		if not idx or creatureName == MI2_Target.name then
			idx = MI2_Target.index;
		end

		if idx then
			MI2_RecordKill( idx, lPlayer, tonumber(xp) );
			lLastRecordedKill = "";
		end
	end

end  -- of MI2_EventCreatureDiesXP()


-----------------------------------------------------------------------------
-- MI2_CreatureDiesHostile()
--
-- event handler for chat message telling me that a hostile creature in
-- my vicinity has died, this message will only get processed if my current
-- target is identical to the last creature that attacked me
--
-- reason: if the crature which I am fighting dies I will still have it
-- as last target when this chat message event is issued
-----------------------------------------------------------------------------
function MI2_CreatureDiesHostile()
	local index = MI2_Target.mobIndex;

	if index and UnitIsDead("target") and index ~= lLastRecordedKill then
		if MI2_Debug > 0 then chattext( "M2DBG: hostile mob died : arg1=["..(arg1 or "nil").."], target=["..(index or "nil").."], lastKill=["..(lLastRecordedKill or "nil").."]" ); end

		for creatureName in string.gfind(arg1, MI_MOB_DIES_WITHOUT_XP ) do
			if  creatureName == MI2_Target.name  then
				MI2_RecordKill( index, lPlayer, 0 );
				lLastRecordedKill = index;
			end
		end
	end
end  -- of MI2_CreatureDiesHostile()


-----------------------------------------------------------------------------
-- MI2_EventUnitHealth()
--
-- event handler for unit health event
-- first call MobHealth event handler, then check for change to health value
-- if health value has changed update game tooltip
-----------------------------------------------------------------------------
function MI2_EventUnitHealth()
	local updated = MI2_MH_EventUnitHealth();
	if UnitIsUnit("target","mouseover") and updated and lMiHealthLine then
		local healthLine = getglobal("GameTooltipTextRight"..lMiHealthLine);
		local healthText = MI2_GetMobHealthStr( lMiMouseoverIndex, UnitHealth("mouseover") )
		if healthText then
			healthLine:SetText( healthText );
		end
	end
end  -- of MI2_EventRegenDisabled()


-----------------------------------------------------------------------------
-- MI2_UpdateMobInfoState()
--
-- Enable or disable all Mob ToolTip options depending on state of
-- "DisableMobInfo". Update event handlers accordingly.
-----------------------------------------------------------------------------
function MI2_UpdateMobInfoState()
	local children = { MI2_FrmTooltipOptions:GetChildren() };

	-- update MobInfo options dialog
	for index, frame in children do
		--chattext( "DBG: child index = ["..index.."], text=["..tostring(frame:GetName()).."]" );
		if frame ~= MI2_OptDisableMobInfo then
			if MobInfoConfig.DisableMobInfo == 0 then
				frame:Enable();
				getglobal(frame:GetName().."Text"):SetTextColor( 1.0, 0.8, 0.0 );
			else
				frame:Disable();
				getglobal(frame:GetName().."Text"):SetTextColor( 0.5, 0.5, 0.5 );
			end
		end
	end

	lEventHandlers = {};
	lEventHandlers["PLAYER_TARGET_CHANGED"] = MI2_OnTargetChanged;
	lEventHandlers["VARIABLES_LOADED"] = MI2_MobInfo_Initialize;

	-- register event handlers for tooltip (if active)
	if MobInfoConfig.DisableMobInfo == 0 then
		if MI2_Debug > 0 then chattext( "M2DBG: UpdateMobInfoState: MobInfo enabled" ); end
		lEventHandlers["CHAT_MSG_COMBAT_XP_GAIN"] = MI2_EventCreatureDiesXP;
		lEventHandlers["CHAT_MSG_COMBAT_HOSTILE_DEATH"] = MI2_CreatureDiesHostile;
		lEventHandlers["LOOT_OPENED"] = MI2_EventLootOpened;
		lEventHandlers["LOOT_CLOSED"] = MI2_EventLootClosed;
		lEventHandlers["LOOT_SLOT_CLEARED"] = MI2_EventLootSlotCleared;
		lEventHandlers["CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE"] = MI2_EventSpellDamage;
		lEventHandlers["CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS"] = MI2_EventMeleeDamage;
		lEventHandlers["CHAT_MSG_COMBAT_SELF_HITS"] = MI2_EventSelfMelee;
		lEventHandlers["CHAT_MSG_SPELL_SELF_DAMAGE"] = MI2_EventSelfSpell;
		lEventHandlers["CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE"] = MI2_EventSelfSpellPeriodic;
		lEventHandlers["CHAT_MSG_COMBAT_PET_HITS"] = MI2_EventSelfPet;
		lEventHandlers["CHAT_MSG_SPELL_PET_DAMAGE"] = MI2_EventSelfPet;
	end

	-- register event handlers for MobHealth (if active)
	if MobInfoConfig.DisableHealth == 0 then
		if MI2_Debug > 0 then chattext( "M2DBG: UpdateMobInfoState: MobHealth enabled" ); end
		lEventHandlers["UNIT_COMBAT"] = MI2_MH_EventUnitCombat;
		lEventHandlers["UNIT_HEALTH"] = MI2_EventUnitHealth;
	end
end  -- MI2_UpdateMobInfoState()


-----------------------------------------------------------------------------
-- MI2_OnLoad()
--
-- register all events that we want to receive and process, build table
-- of event handler functions for easy processing of events in "OnEvent"
-----------------------------------------------------------------------------
function MI2_OnLoad()
	-- register all events that we want to catch and process
	this:RegisterEvent("CHAT_MSG_COMBAT_XP_GAIN");
	this:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	this:RegisterEvent("LOOT_OPENED");
	this:RegisterEvent("LOOT_CLOSED");
	this:RegisterEvent("LOOT_SLOT_CLEARED");
	this:RegisterEvent("PLAYER_TARGET_CHANGED");
	this:RegisterEvent("PLAYER_REGEN_DISABLED");
	this:RegisterEvent("VARIABLES_LOADED");
	this:RegisterEvent("UNIT_COMBAT");
	this:RegisterEvent("UNIT_HEALTH");
	this:RegisterEvent("CHAT_MSG_SPELL_CREATURE_VS_SELF_DAMAGE");
	this:RegisterEvent("CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS");
	this:RegisterEvent("CHAT_MSG_COMBAT_SELF_HITS");
	this:RegisterEvent("CHAT_MSG_SPELL_SELF_DAMAGE");
	this:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_CREATURE_DAMAGE");
	this:RegisterEvent("CHAT_MSG_COMBAT_PET_HITS");
	this:RegisterEvent("CHAT_MSG_SPELL_PET_DAMAGE");

	-- build event handler table for easy event processing (see "MI2_OnEvent()")
	lEventHandlers["VARIABLES_LOADED"] = MI2_MobInfo_Initialize;

	-- set some stuff that is needed (only) for improved compatibility
	-- to other AddOns wanting to use MobHealth info
	if  not MobHealth_OnEvent  then
		if MI2_Debug > 0 then chattext( "M2DBG: setting up compatibility variables" ); end
		MobHealthFrame = "MI2";
		MobHealth_OnEvent = MI2_OnEvent;
	end

	if MI2_Debug > 0 then chattext( "M2DBG: MobInfo_OnLoad: all events registered" ); end
end  -- of MI2_OnLoad()


-----------------------------------------------------------------------------
-- MI2_OnEvent()
--
-- MobInfo main event handler function, gets called for all registered events
-- uses table of event handlers which gets initialised in "OnLoad"
-----------------------------------------------------------------------------
function MI2_OnEvent( event )	
	if event then
		-- debug output section for testing/debugging
		if MI2_Debug > 2 then
			chattext("M2DBG: event="..event..", a1="..(arg1 or "<nil>")..", a2="..(arg2 or "<nil>")..", a3="..(arg3 or "<nil>")..", a4="..(arg4 or "<nil>"));
		end

		-- call event handler function for event
		if  lEventHandlers[event]  then
			lEventHandlers[event]();
			return 0;
		end
	end
end  -- of MI2_OnEvent

